/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
#ifndef INC_ARR_h
#define INC_ARR_h
/*	@(#)ARR.h	1.7 8/24/92
 *	Author:	Roger A. Cole
 *	Date:	02-08-91
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 *	Copyright 1991-92, the Regents of the University of California,
 *	and the University of Chicago Board of Governors.
 *
 *	This software was produced under  U.S. Government contracts:
 *	(W-7405-ENG-36) at the Los Alamos National Laboratory,
 *	and (W-31-109-ENG-38) at Argonne National Laboratory.
 *
 *	Initial development by:
 *		The Controls and Automation Group (AT-8)
 *		Ground Test Accelerator
 *		Accelerator Technology Division
 *		Los Alamos National Laboratory
 *
 *	Co-developed with
 *		The Controls and Computing Group
 *		Accelerator Systems Division
 *		Advanced Photon Source
 *		Argonne National Laboratory
 *
 * Modification Log:
 * -----------------
 *  .00 02-08-91 rac	initial version
 *  .01 07-31-91 rac	installed in SCCS
 *  .02 08-05-91 rac	allow choosing `by channel' file
 *  .03 10-15-91 rac	store default printer name in context
 *  .04 11-25-91 rac	remove `set printer' command frame
 *  .05 03-01-92 rac	add more default items to context
 *  .06 04-07-92 rac	split for sharing with AR; add choice for using
 *			data values or statistics; add fields for restricted
 *			time
 * ARR.h,v
 * Revision 1.3  2002/08/02 15:37:50  jba
 * Added license information.
 *
 * Revision 1.2  1995/04/05 13:57:34  jba
 * Portability and ansi changes.
 *
 * Revision 1.1  1995/02/13  15:32:30  jba
 * Modified for unbundling of libppr and syd files to ar/syd
 *
 * Revision 1.2  1994/05/06  19:05:19  mrk
 * conversion from sccs to cvs
 *
 *
 */
/*+/mod***********************************************************************
* TITLE	ARR.h - definitions for ARR, the AR retrieval utility
*
* DESCRIPTION
*
*-***************************************************************************/
#include <guiSubr.h>

#define ArrMax(val1, val2) ((val1)>(val2) ? (val1) : (val2))
#define ArrMin(val1, val2) ((val1)<(val2) ? (val1) : (val2))


/*-----------------------------------------------------------------------------
* ARR_CHAN
*
*	the name is of the form "f name", where 'f' is ' ' or 'F' depending
*	on whether the channel is treated as synchronous or filled,
*	respectively.
*----------------------------------------------------------------------------*/
typedef struct arrChan {
    struct arrChan *pNext;	/* next channel in list */
    struct arrChan *pPrev;	/* previous channel in list */
    SYD_CHAN	*pSChan;	/* pointer to sync data chan struct */
    char	name[db_name_dim+2];/* flag and channel name */
    Menu_item	mi;		/* menu item for channel */
    int		filled;		/* channel is used in "filled" mode */
} ARR_CHAN;

struct arrRetCtx;
struct exCtx;
struct plCtx;
struct prCtx;

/*/subhead ARR_CTX-------------------------------------------------------------
*
*	These all start out with zero values, thanks to the memset call in
*	main.
*----------------------------------------------------------------------------*/
typedef struct {
    GUI_CTX	guiCtx;
    GUI_PRT	*pGuiPrt;	/* pointer to gui printer context */
    struct exCtx *pExCtx;	/* pointer to export context */
    struct plCtx *pPlCtx;	/* pointer to plotting context */
    struct prCtx *pPrCtx;	/* pointer to printer context */

    struct arrRetCtx *pRetCtx;
    ARR_CHAN	*pChanHead;	/* pointer to first channel */
    ARR_CHAN	*pChanTail;	/* pointer to last channel */
    SYD_SPEC	*pSspec;	/* sync data spec */
    Panel_item	useStats_PC;	/* panel choice item for "use statistics" */
    Panel_item	restrict_PC;	/* panel choice for "restricted time" */
    TS_STAMP	restrictBegTs;	/* beginning of restricted time range */
    TS_STAMP	restrictEndTs;	/* end of restricted time range */
    char	*dfltSSF_dir;	/* default sample set file dir */
    char	*dfltBCF_dir;	/* default by channel file dir */
    char	*dfltRet_dir;	/* default retrieval file dir */
    char	*dfltExp_dir;	/* default export file dir */

} ARR_CTX;

/*-----------------------------------------------------------------------------
* context pointer
*----------------------------------------------------------------------------*/
extern ARR_CTX	arrCtx;	/* this is global for the benefit of event handlers,
			as well as guiShellCmd and friends */


void RETinit(), RETquit();
void RETbcfLsDone();
void RETssfLsDone();

#endif	/* INC_ARR_h */
