/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
/*	@(#)PL.c	1.4 2/23/93
 *	Author:	Roger A. Cole
 *	Date:	02-08-91
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 */
/*+/mod***********************************************************************
* TITLE	PL.c - plotting routines for ARR and AR
*
* DESCRIPTION
*
*-***************************************************************************/
#include <stdio.h>

#define XVIEW
#if defined XVIEW
#   include <xview/xview.h>
#   include <xview/cms.h>
#   include <xview/frame.h>
#   include <xview/notice.h>
#   include <xview/panel.h>
#   include <xview/textsw.h>
#endif

#include <genDefs.h>
#include <guiSubr.h>
#include <arAccessLib.h>
#include <sydDefs.h>
#include <sydPlotDefs.h>
#include <db_access.h>
#include <pprPlotDefs.h>
#include "enumShared.h"
#include "PL.h"


/*+/subr**********************************************************************
* NAME	PLwinEvent - handle events for plot window
*
*	WIN_REPAINT
*	o   repaint the plot
*	o   reset the prior value for `time cursor' time stamp
*
*	MS_LEFT up
*	o   find the channel whose grid or annotation the mouse is at,
*	    put that channel's name in the setup window, and update
*	    the value display for that channel.
*
*-*/
static void
PLwinEvent(canvas,event)
Canvas	canvas;
Event *event;
{   PL_CTX	*pPlCtx=(PL_CTX *)xv_get(canvas, XV_KEY_DATA, PLCTX);
    int		x,y;	/* dummy for position */
    long	stat;
    SYD_PL_SLAVE *pSlave;

    if (event_id(event) == WIN_REPAINT) {
	if (pPlCtx->PLplotting) {
	    stat = sydPlotWinReplot(&pPlCtx->PLmaster);
	    assertAlways(stat == S_syd_OK);
	    pPlCtx->PLrestBegTs.secPastEpoch = 0;
	    pPlCtx->PLrestEndTs.secPastEpoch = 0;
	}
    }
    else if (event_id(event) == MS_LEFT && event_is_up(event)) {
	int	valueB, valueE;
	TS_STAMP tsB, tsE;	/* time stamp at begin and end sliders */
	char	textB[28], textE[28];/* text date/time at begin and end */
	char	*pTextB=textB, *pTextE=textE;
	int	first, last;	/* subscripts for first and last in set */
	double	range;		/* seconds between first and last */
	double	deltaB, deltaE;	/* delta sec for begin and end sliders */
	SYD_SPEC *pSspec;

	if (pPlCtx->ppSspec == NULL)
	    return;
	pSspec=*pPlCtx->ppSspec;
	first = pSspec->restrictFirstData;
	if (first < 0)
	    return;
	last = pSspec->restrictLastData;
	range = pSspec->pDeltaSec[last] - pSspec->pDeltaSec[first];

	strcpy(textB, (char *)xv_get(pPlCtx->PLrestB_PT, PANEL_VALUE));
	if (strlen(textB) > (size_t)0 && tsTextToStamp(&tsB, &pTextB) == S_ts_OK) {
	    TsDiffAsDouble(&deltaB, &tsB, &pSspec->pTimeStamp[first]);
	    valueB = deltaB / range * 1000.;
	    if (valueB < 0)		valueB = 0;
	    else if (valueB > 999)	valueB = 999;
	}
	else
	    valueB = 0;
	xv_set(pPlCtx->PLrestB_PSL, PANEL_VALUE, valueB, NULL);
	strcpy(textE, (char *)xv_get(pPlCtx->PLrestE_PT, PANEL_VALUE));
	if (strlen(textE) > (size_t)0 && tsTextToStamp(&tsE, &pTextE) == S_ts_OK) {
	    TsDiffAsDouble(&deltaE, &tsE, &pSspec->pTimeStamp[first]);
	    valueE = deltaE / range * 1000.;
	    if (valueE < 1)		valueE = 1;
	    else if (valueE > 1000)	valueE = 1000;
	}
	else
	    valueE = 1000;
	xv_set(pPlCtx->PLrestE_PSL, PANEL_VALUE, valueE, NULL);
	x = event_x(event);
	y = event_y(event);
	pSlave = sydPlotChan_event(pPlCtx->PLmaster, x, y);
	if (pSlave != NULL) {
	    guiTextFieldPrintf(pPlCtx->PLrestChan_PT, "%s",
						pSlave->pSChan->name);
	    PLrestrict_show(pPlCtx, valueB, valueE);
	}
    }
}

/*+/subr**********************************************************************
* NAME	PLwinPlot_xvo - do a plot
*
* NOTES
* 1.	Under XView, the plot frame already exists and a repaint event has
*	already occurred, even as the frame was created.  Because of this,
*	the first repaint is done here, manually.  Succeeding repaints for
*	the plot are triggered by the event handler.
*
*-*/
void PLwinPlot_xvo();
static void PLwinPlot_mi(menu, item)
Menu menu;Menu_item item;{PLwinPlot_xvo(item);}
static void PLwinPlot_xvo(item)
Xv_opaque item;
{   PL_CTX	*pPlCtx=(PL_CTX *)xv_get(item, XV_KEY_DATA, PLCTX);
    PL_TYPE	cmdType=(PL_TYPE)xv_get(item, XV_KEY_DATA, PLTYPE);
    SYD_SPEC	*pSspec=*pPlCtx->ppSspec;
    Window	plotWindow;
    GC		plotGC;
    SYD_PL_SLAVE	*pSlave;
    long	stat;
    SYD_CHAN	*pSChan;
    TS_STAMP	now;
    char	nowText[32], begText[28], endText[28];
    char	title[120];
    char	command[160];
    char	resultBuf[80];
    int		type, point, mark, show, with, line, autoScale, bars;
    SYD_PLAX	plotType;
    SYD_PL_MSTR master, *pMstr=&master;
    char	HCname[20];
    double	ymin=0., ymax=0.;

    guiCFshowPin(pPlCtx->PLwin_CF);
    if (pPlCtx->PLplotting && cmdType == PL_PLAIN) {
	pPlCtx->PLplotting = 0;
	stat = sydPlotDone(&pPlCtx->PLmaster, 1);
	assertAlways(stat == S_syd_OK);
    }
    if (pSspec == NULL || pSspec->chanCount <= 0) {
	if (cmdType == PL_NULL)
	    return;
	guiNotice(pPlCtx->pGuiCtx, "no channels selected");
	return;
    }
    strcpy(HCname, (char *)xv_get(pPlCtx->pGuiPrt->printer_PT, PANEL_VALUE));

    type = 2 * (int)xv_get(pPlCtx->PLhoriz_PCS, PANEL_VALUE);
    if ((int)xv_get(pPlCtx->PLseparate_PCS, PANEL_VALUE) == 1)
	type++;
    if (type == 0)	plotType = SYD_PLAX_TY;
    else if (type == 1)	plotType = SYD_PLAX_TYY;
    else if (type == 2)	plotType = SYD_PLAX_XY;
    else if (type == 3)	plotType = SYD_PLAX_XYY;
    else if (type == 4)	plotType = SYD_PLAX_STRIP_Y;
    else if (type == 5)	plotType = SYD_PLAX_STRIP_YY;
    else if (type == 6)	plotType = SYD_PLAX_Y;
    else if (type == 7)	plotType = SYD_PLAX_YY;
    if (plotType == SYD_PLAX_XY || plotType == SYD_PLAX_XYY) {
	if (pSspec->chanCount < 2) {
	    if (cmdType == PL_NULL)
		return;
	    guiNotice(pPlCtx->pGuiCtx, "no channel to plot on X axis");
	    return;
	}
    }
    if (pSspec->sampleCount <= 0) {
	if (cmdType == PL_NULL)
	    return;
	guiNotice(pPlCtx->pGuiCtx, "no samples to plot; use RETRIEVE");
	return;
    }
    line = 1;
    point = mark = 0;
    with = (int)xv_get(pPlCtx->PLwith_PCS, PANEL_VALUE);
    if (with == 1)	mark = 1;
    else if (with == 2)	mark = 1, line = 0;
    else if (with == 3)	point = 1, line = 0;
    show = (int)xv_get(pPlCtx->PLstat_PCS, PANEL_VALUE);
    autoScale = (int)xv_get(pPlCtx->PLscale_PCS, PANEL_VALUE);
    bars = (int)xv_get(pPlCtx->PLbars_PCS, PANEL_VALUE);

    if (pPlCtx->useStats == 0 && pPlCtx->useMeans == 0) {
	sydSetAttr(pSspec, SYD_ATTR_USE_STATS, 0, NULL);
	bars = 0;
    }
    else {
	if (plotType == SYD_PLAX_STRIP_Y || plotType == SYD_PLAX_STRIP_YY) {
	    guiNotice(pPlCtx->pGuiCtx, "can't do stats for strip charts");
	    return;
	}
	if (pPlCtx->useStats == 1)
	    sydSetAttr(pSspec, SYD_ATTR_USE_STATS, 1, NULL);
	else
	    sydSetAttr(pSspec, SYD_ATTR_USE_MEANS, 1, NULL);
	sydSampleSetStats(pSspec);
    }
    if (plotType == SYD_PLAX_STRIP_Y || plotType == SYD_PLAX_STRIP_YY) {
	float	delay;
	if (sscanf((char *)xv_get(pPlCtx->stripDelay_PT, PANEL_VALUE),
							"%f", &delay) != 1 ||
		delay <= 0. || delay > 1000.) {
	    guiNotice(pPlCtx->pGuiCtx, "illegal strip chart delay");
	    return;
	}
	pPlCtx->stripDelay = delay;
	pMstr->stripIncr = delay;
	pPlCtx->stripLastTs.secPastEpoch = 0;
    }

    if (autoScale == 2) {
/*-----------------------------------------------------------------------------
*	if using `yes, shared' for scaling, find the `global' min and max
*----------------------------------------------------------------------------*/
	for (pSChan=pSspec->pChanHead; pSChan!=NULL; pSChan = pSChan->pNext) {
	    if (pSChan->dataChan && pSChan->dbrType != DBR_TIME_ENUM) {
		if (ymin == ymax) {
		    ymin = pSChan->restrictMinDataVal;
		    ymax = pSChan->restrictMaxDataVal;
		}
		else {
		    if (pSChan->restrictMinDataVal < ymin)
			ymin = pSChan->restrictMinDataVal;
		    if (pSChan->restrictMaxDataVal > ymax)
			ymax = pSChan->restrictMaxDataVal;
		}
	    }
	}
    }
    else if (autoScale == 1)
	ymin = ymax = 0.;

    (void)tsLocalTime(&now);
    (void)tsStampToText(&now, TS_TEXT_MONDDYYYY, nowText);
    (void)tsStampToText(&pSspec->pTimeStamp[pSspec->firstData], TS_TEXT_MMDDYY,
								begText);
    (void)tsStampToText(&pSspec->pTimeStamp[pSspec->lastData], TS_TEXT_MMDDYY,
								endText);
    (void)sprintf(title, "samples for %s through %s", begText, endText);

    if (cmdType == PL_PLAIN || cmdType == PL_NULL) {
	pPlCtx->PLplotting = 1;
	plotWindow =
	    (Window)xv_get(canvas_paint_window(pPlCtx->PLwinCanvas), XV_XID);
	plotGC = DefaultGC(pPlCtx->pGuiCtx->pDisp,
				DefaultScreen(pPlCtx->pGuiCtx->pDisp));
	XSetBackground(pPlCtx->pGuiCtx->pDisp, plotGC, pPlCtx->PLwinBg);
	XSetForeground(pPlCtx->pGuiCtx->pDisp, plotGC, pPlCtx->PLwinFg);

	stat = sydPlotInitUW(&pPlCtx->PLmaster, pSspec,
		    		pPlCtx->pGuiCtx->pDisp, plotWindow, plotGC);
	assertAlways(stat == S_syd_OK);
	stat = sydPlotSetTitles(&pPlCtx->PLmaster, title, NULL, NULL, NULL);
	assertAlways(stat == S_syd_OK);
	pPlCtx->PLmaster.plotAxis = plotType;
	pPlCtx->PLmaster.linePlot = line;
	pPlCtx->PLmaster.markPlot = mark;
	pPlCtx->PLmaster.pointPlot = point;
	pPlCtx->PLmaster.showStat = show;
	sydPlotSetAttr(&pPlCtx->PLmaster, SYD_PLATTR_STDDEV, bars, NULL);
	for (pSChan=pSspec->pChanHead; pSChan!=NULL; pSChan = pSChan->pNext) {
	    if (pSChan->dataChan) {
		pSlave = sydPlotChanAdd(&pPlCtx->PLmaster, pSChan);
		assert(pSlave != NULL);
		if (autoScale)
		    sydPlotAxisAutoRange(pSlave, ymin, ymax);
	    }
	}
	stat = sydPlotWinReplot(&pPlCtx->PLmaster);
	assertAlways(stat == S_syd_OK);
	pPlCtx->PLrestBegTs.secPastEpoch = 0;
	pPlCtx->PLrestEndTs.secPastEpoch = 0;
    }
    else if (cmdType == PL_EPS) {
	FILE	*f;
	int	answer;
	char	path[2*GUI_TDIM];
	int	modeCode;

	strcpy(pPlCtx->dir, (char *)xv_get(pPlCtx->dir_PT, PANEL_VALUE));
	strcpy(pPlCtx->file, (char *)xv_get(pPlCtx->file_PT, PANEL_VALUE));
	modeCode = (int)xv_get(pPlCtx->fileMode_PCS, PANEL_VALUE);
	if (strlen(pPlCtx->file) == 0) {
	    guiNotice(pPlCtx->pGuiCtx,
		"pick an EPS file using the \"plotting setup\" window");
	    guiCFshowPin(pPlCtx->PLsetup_CF);
	    return;
	}
	sprintf(path, "%s/%s", pPlCtx->dir, pPlCtx->file);
        if (modeCode == 0) {		/* create mode */
	    if ((f = fopen(path, "r")) != NULL) {
		guiNoticePrintf(pPlCtx->pGuiCtx, "%s already exists", path);
		return;
	    }
	    fclose(f);
	    if ((f = fopen(path, "w")) == NULL) {
		guiNoticeFile(pPlCtx->pGuiCtx, "Can't open for output", path);
		return;
	    }
	}
        else if (modeCode == 1) {	/* append mode */
	    if ((f = fopen(path, "a")) == NULL) {
		guiNoticeFile(pPlCtx->pGuiCtx, "Can't open for output", path);
		return;
	    }
	}
	else {				/* over-write mode */
	    if ((f = fopen(path, "w")) == NULL) {
		guiNoticeFile(pPlCtx->pGuiCtx,"Unable to open for output",path);
		return;
	    }
	}

	if (pPlCtx->PLplotting) {
	    master.width = pPlCtx->PLmaster.width;
	    master.height = pPlCtx->PLmaster.height;
	}
	else {
	    master.width = 512;
	    master.height = 512;
	}
	stat = sydPlotInit(&master, pSspec, PPR_WIN_EPS, path, NULL, 0);
	assertAlways(stat == S_syd_OK);
	stat = sydPlotSetTitles(&master, title, NULL, NULL, NULL);
	assertAlways(stat == S_syd_OK);
	master.plotAxis = plotType;
	master.linePlot = line;
	master.markPlot = mark;
	master.pointPlot = point;
	master.showStat = show;
	sydPlotSetAttr(pMstr, SYD_PLATTR_STDDEV, bars, NULL);
	for (pSChan=pSspec->pChanHead; pSChan!=NULL; pSChan = pSChan->pNext) {
	    if (pSChan->dataChan) {
		pSlave = sydPlotChanAdd(&master, pSChan);
		assert(pSlave != NULL);
		if (autoScale)
		    sydPlotAxisAutoRange(pSlave, ymin, ymax);
	    }
	}

	stat = sydPlotWinLoop(&master);
	assert(stat == S_syd_OK);

	stat = sydPlotDone(&master, 1);
	assert(stat == S_syd_OK);
    }
    else if (cmdType == PL_HC) {
	char	command[120];

	if (strlen(HCname) == 0) {
	    guiNotice(pPlCtx->pGuiCtx,
		"pick a printer; use \"printer setup...\" on \"Plot\" button");
	    return;
	}
	sprintf(pPlCtx->HCfName, "/tmp/ARR%d.HC.tmp", getpid());
	if (pPlCtx->PLplotting) {
	    master.width = pPlCtx->PLmaster.width;
	    master.height = pPlCtx->PLmaster.height;
	}
	else {
	    master.width = 512;
	    master.height = 512;
	}
	stat = sydPlotInit(&master, pSspec,
				PPR_WIN_POSTSCRIPT, pPlCtx->HCfName, NULL, 0);
	assertAlways(stat == S_syd_OK);
	stat = sydPlotSetTitles(&master, title, NULL, NULL, NULL);
	assertAlways(stat == S_syd_OK);
	master.plotAxis = plotType;
	master.linePlot = line;
	master.markPlot = mark;
	master.pointPlot = point;
	master.showStat = show;
	sydPlotSetAttr(pMstr, SYD_PLATTR_STDDEV, bars, NULL);
	for (pSChan=pSspec->pChanHead; pSChan!=NULL; pSChan = pSChan->pNext) {
	    if (pSChan->dataChan) {
		pSlave = sydPlotChanAdd(&master, pSChan);
		assert(pSlave != NULL);
		if (autoScale)
		    sydPlotAxisAutoRange(pSlave, ymin, ymax);
	    }
	}

	stat = sydPlotWinLoop(&master);
	assert(stat == S_syd_OK);

	stat = sydPlotDone(&master, 1);
	assert(stat == S_syd_OK);
	if (cmdType == PL_HC) {
	    sprintf(command, "lpr -P%s %s", HCname, pPlCtx->HCfName);
	    guiShellCmd(pPlCtx->pGuiCtx, command, lprClient, NULL, NULL);
	}
    }
}

/*+/subr**********************************************************************
* NAME	PLsetupInitCF
*
*	o   "plot with" choice stack, to choose line, point, or mark plots
*	o   "horiz vs vert" choice stack, to choose layout and use of axes
*	o   "value status" choice stack, to choose whether to suppress or
*	    show the status of individual points on plots
*
*-*/
static void
PLsetupInitCF(pPlCtx, title)
PL_CTX	*pPlCtx;
char	*title;
{
    Panel	panel;
    int		x=5, y=5, ht=0, x1;
    Xv_opaque	item;
    char	msg[GUI_TDIM];
    void	PLrestrictB_xvo(), PLrestrictE_xvo();
    void	PLrestrictApply_pb(), PLrestrictReset_pb();

    sprintf(msg, "%s: setup", title);
    pPlCtx->PLsetup_CF=guiFrameCmd(msg,pPlCtx->pGuiCtx->baseFrame,&panel,NULL);

/*-----------------------------------------------------------------------------
* plot with	lines  lines and marks  marks  points
* vertical axes: separate  shared
* overlay:	grid
* horiz axis:	time  value  strip chart  bin number
* strip chart:  run  freeze   delay _____ seconds
* value status:	suppress  show
* auto scale:	no  yes
* error bars:    none  stdDev
*
* Apply  Reset            start                    end
*                   (=========[]=========)  (=========[]=========)
* restricted time:  ______________________  ______________________
* channel value:    ______________________  ______________________
* channel name:  ________________________________
*
* EPS directory: _______________________
* EPS file: _______________________  V create
*                                      append
*                                      over-write
*----------------------------------------------------------------------------*/

    x1 = 100;
    guiMessage("plot with:", panel, &x,&y,&ht);
    pPlCtx->PLwith_PCS = guiChoice("",
		"lines","lines and marks","marks","points", NULL,
		panel, 1, 1, 0, &x1,&y,&ht, NULL, 0,NULL,0,NULL);
    x1 = 100; y += ht + 5; x = 5; ht = 0;
    guiMessage("vertical axes:", panel, &x,&y,&ht);
    pPlCtx->PLseparate_PCS = guiChoice("", "separate", "shared", NULL,
		panel, 1, 1, 0, &x1,&y,&ht, NULL, 0,NULL,0,NULL);
    x1 = 100; y += ht + 5; x = 5; ht = 0;
    guiMessage("overlay:", panel, &x,&y,&ht);
    pPlCtx->PLoverlay_PCS = guiChoice("", "grid", NULL,
		panel, 1, 1, 0, &x1,&y,&ht, NULL, 0,NULL,0,NULL);

    x1 = 100; y += ht + 5; x = 5; ht = 0;
    guiMessage("horiz axis:", panel, &x,&y,&ht);
    pPlCtx->PLhoriz_PCS = guiChoice("",
		"time", "value", "strip chart", "bin number", NULL,
		panel, 1, 1, 0, &x1,&y,&ht, NULL, 0,NULL,0,NULL);
    x1 = 110; y += ht + 5; x = x1; ht = 0;
    guiMessage("strip chart", panel, &x,&y,&ht);
    pPlCtx->stripMode_PCS = guiChoice("", "run", "freeze", NULL,
		panel, 1, 1, 0, &x,&y,&ht, NULL, 0,NULL,0,NULL);
    x += 15;
    guiMessage("delay", panel, &x,&y,&ht);
    pPlCtx->stripDelay_PT = guiTextField("", panel, &x,&y,&ht,
		NULL, 10, 10, 0,NULL,0,NULL);
    guiMessage("seconds", panel, &x,&y,&ht);
    x1 = 100; y += ht + 5; x = 5; ht = 0;
    guiMessage("value status:", panel, &x,&y,&ht);
    pPlCtx->PLstat_PCS = guiChoice("",
		"suppress", "show", NULL,
		panel, 1, 1, 0, &x1,&y,&ht, NULL, 0,NULL,0,NULL);
    x1 = 100; y += ht + 5; x = 5; ht = 0;
    guiMessage("auto scale:", panel, &x,&y,&ht);
    pPlCtx->PLscale_PCS = guiChoice("",
		"no", "yes", "yes, shared", NULL,
		panel, 1, 1, 0, &x1,&y,&ht, NULL, 0,NULL,0,NULL);
    x1 = 100; y += ht + 5; x = 5; ht = 0;
    guiMessage("error bars:", panel, &x,&y,&ht);
    pPlCtx->PLbars_PCS = guiChoice("",
		"none", "stdDev", NULL,
		panel, 1, 1, 0, &x1,&y,&ht, NULL, 0,NULL,0,NULL);
/*-----------------------------------------------------------------------------
*	time restriction stuff
*----------------------------------------------------------------------------*/
    x1 = 180; y += ht + 25; x = 5; ht = 0;
    pPlCtx->PLrestApply_PB = guiButton("Apply", panel, &x,&y,&ht,
		PLrestrictApply_pb, NULL, PLCTX, pPlCtx, PLTYPE, PL_PLAIN);
    pPlCtx->PLrestReset_PB = guiButton("Reset", panel, &x,&y,&ht,
		PLrestrictReset_pb, NULL, PLCTX, pPlCtx, PLTYPE, PL_PLAIN);
    guiMessage("start", panel, &x1,&y,&ht);
    x1 = 380;
    guiMessage("end", panel, &x1,&y,&ht);
    x1 = 100; y += ht + 5; x = 5; ht = 0;
    pPlCtx->PLrestB_PSL = item = (Panel_item)xv_create(panel, PANEL_SLIDER,
		PANEL_VALUE, 0, PANEL_MIN_VALUE, 0, PANEL_MAX_VALUE, 999,
		PANEL_SLIDER_WIDTH, 170, XV_X, x1, XV_Y, y,
		PANEL_SHOW_VALUE, FALSE, PANEL_SHOW_RANGE, FALSE,
		PANEL_NOTIFY_PROC, PLrestrictB_xvo,
		PANEL_NOTIFY_LEVEL, PANEL_ALL,
		XV_KEY_DATA, PLCTX, pPlCtx, NULL);
    x1 = 305;
    pPlCtx->PLrestE_PSL = item = (Panel_item)xv_create(panel, PANEL_SLIDER,
		PANEL_VALUE, 1000, PANEL_MIN_VALUE, 1, PANEL_MAX_VALUE, 1000,
		PANEL_SLIDER_WIDTH, 170, XV_X, x1, XV_Y, y,
		PANEL_SHOW_VALUE, FALSE, PANEL_SHOW_RANGE, FALSE,
		PANEL_NOTIFY_PROC, PLrestrictE_xvo,
		PANEL_NOTIFY_LEVEL, PANEL_ALL,
		XV_KEY_DATA, PLCTX, pPlCtx, NULL);
    ht = xv_get(item, XV_HEIGHT);
    x1 = 100; y += ht + 5; x = 5; ht = 0;
    guiMessage("restricted time:", panel, &x,&y,&ht);
    pPlCtx->PLrestB_PT = guiTextField("", panel, &x1,&y,&ht, NULL, 21, 21,
			0,NULL,0,NULL);
    x1 = 300;
    pPlCtx->PLrestE_PT = guiTextField("", panel, &x1,&y,&ht, NULL, 21, 21,
			0,NULL,0,NULL);
    xv_set(pPlCtx->PLrestE_PT, PANEL_VALUE, "", NULL);
    x1 = 100; y += ht + 5; x = 5; ht = 0;
    guiMessage("channel value:", panel, &x,&y,&ht);
    pPlCtx->PLrestBval_PT = guiTextField("", panel, &x1,&y,&ht, NULL, 21, 21,
			0,NULL,0,NULL);
    x1 = 300;
    pPlCtx->PLrestEval_PT = guiTextField("", panel, &x1,&y,&ht, NULL, 21, 21,
			0,NULL,0,NULL);
    x1 = 100; y += ht + 5; x = 5; ht = 0;
    guiMessage("channel name:", panel, &x,&y,&ht);
    pPlCtx->PLrestChan_PT = guiTextField("", panel, &x1,&y,&ht, NULL, 40, 40,
			0,NULL,0,NULL);
/*-----------------------------------------------------------------------------
*	EPS file stuff
*----------------------------------------------------------------------------*/
    y += ht + 25; x = 5; ht = 0;
    pPlCtx->dir_PT = guiTextField("EPS directory:", panel, &x,&y,&ht, NULL,
		GUI_TDIM-1, 40, 0, NULL, 0, NULL);
    sprintf(pPlCtx->dir, "%s/data", pPlCtx->pGuiCtx->pwd);
    xv_set(pPlCtx->dir_PT, PANEL_VALUE, pPlCtx->dir, NULL);
    y += ht + 5; x = 5; ht = 0;

    pPlCtx->file_PT = guiTextField("EPS file:", panel, &x,&y,&ht, NULL,
		GUI_TDIM-1, 40, 0, NULL, 0, NULL);
    strcpy(pPlCtx->file, "");
    xv_set(pPlCtx->file_PT, PANEL_VALUE, pPlCtx->file, NULL);
    pPlCtx->fileMode_PCS = item = xv_create(panel, PANEL_CHOICE_STACK,
	PANEL_LAYOUT,		PANEL_VERTICAL,
	PANEL_CHOICE_STRINGS,	"create","append","over-write",NULL,
	PANEL_VALUE,	0,
	XV_X, x, XV_Y, y-5, NULL);
    x += (int)xv_get(item, XV_WIDTH) + 5;
    y += ht + 5; x = 5; ht = 0;

    window_fit(panel);
    window_fit(pPlCtx->PLsetup_CF);
}

/*+/subr**********************************************************************
* NAME	PLrestrictB_xvo
*
*-*/
void
PLrestrictB_xvo(item, value, pEvent)
Panel_item item;
int	value;
Event	*pEvent;
{   PL_CTX	*pPlCtx=(PL_CTX *)xv_get(item, XV_KEY_DATA, PLCTX);
    int		valueE=xv_get(pPlCtx->PLrestE_PSL, PANEL_VALUE);

    if (value > valueE - 10) {
	value = valueE - 10;
	xv_set(pPlCtx->PLrestB_PSL, PANEL_VALUE, value, NULL);
    }
    PLrestrict_show(pPlCtx, value, valueE);
}

void
PLrestrictE_xvo(item, value, pEvent)
Panel_item item;
int	value;
Event	*pEvent;
{   PL_CTX	*pPlCtx=(PL_CTX *)xv_get(item, XV_KEY_DATA, PLCTX);
    int		valueB=xv_get(pPlCtx->PLrestB_PSL, PANEL_VALUE);

    if (value < valueB + 10) {
	value = valueB + 10;
	xv_set(pPlCtx->PLrestE_PSL, PANEL_VALUE, value, NULL);
    }
    PLrestrict_show(pPlCtx, valueB, value);
}

/*+/subr**********************************************************************
* NAME	PLrestrict_show
*
* DESCRIPTION
*	Make visible the result of slider movement.
*
*	The `time range:' item on the main ARR panel is set to `restricted'.
*
*-*/
PLrestrict_show(pPlCtx, valB, valE)
PL_CTX	*pPlCtx;
int	valB, valE;
{
    SYD_SPEC	*pSspec;
    int		first, last;	/* subscripts for first and last in set */
    double	range;		/* seconds between first and last */
    double	deltaB, deltaE;	/* delta sec for begin and end sliders */
    TS_STAMP	tsB, tsE;	/* time stamp at begin and end sliders */
    char	textB[28], textE[28];/* text date/time at begin and end */
    SYD_CHAN	*pSChan;
    char	chan[db_name_dim];
    SYD_PL_SLAVE *pSlave=NULL;

    if (pPlCtx->ppSspec == NULL)
	return;
    pSspec=*pPlCtx->ppSspec;
    if (pSspec == NULL || pSspec->sampleCount <= 0)
	return;
    if (pPlCtx->PLplotting == 0)
	return;

    first = pSspec->restrictFirstData;
    last = pSspec->restrictLastData;
    range = pSspec->pDeltaSec[last] - pSspec->pDeltaSec[first];
    deltaB = ( (double)valB / 1000. ) * range + pSspec->pDeltaSec[first];
    deltaE = ( (double)valE / 1000. ) * range + pSspec->pDeltaSec[first];
    TsAddDouble(&tsB, &pSspec->refTs, deltaB);
    TsAddDouble(&tsE, &pSspec->refTs, deltaE);

/*-----------------------------------------------------------------------------
*	find the channel structure (and the corresponding plot slave) for
*	the channel named on the `setup' window.  Also find the sample numbers
*	corresponding to the start and end stamps and print the values at
*	those two places.
*
*	|    |    |    |    |    |        <---- sample times
*	--------------------------------
*	  |              |                <---- deltaB and deltaE
*	^              ^                  <---- print these samples
*----------------------------------------------------------------------------*/
    strcpy(chan, (char *)xv_get(pPlCtx->PLrestChan_PT, PANEL_VALUE));
    if (strlen(chan) > (size_t)0) {
	pSChan = sydChanFind(pSspec, chan);
	if (pSChan == NULL) {
	    guiTextFieldPrintf(pPlCtx->PLrestBval_PT, "not found");
	    guiTextFieldPrintf(pPlCtx->PLrestEval_PT, "not found");
	}
	else {
	    double	val;
	    char	valTxt[40];
	    int		i, iB=-1, iE=-1;
	    i = first;
	    while (1) {
		if (pSspec->pDeltaSec[i] <= deltaB) iB = i;
		if (pSspec->pDeltaSec[i] <= deltaE) iE = i;
		if (i == last) {
		    if (iB == -1) iB = first;
		    if (iE == -1) iE = last;
		    break;
		}
		if (++i >= pSspec->dataDim)
		    i = 0;
	    }
	    if (pSChan->pFlags[iB].eof)
		strcpy(valTxt, "EOF");
	    else if (pSChan->pFlags[iB].missing)
		strcpy(valTxt, "missing");
	    else {
		if (sydValAsDbl(pSChan, iB, &val))
		    cvtDblToTxt(valTxt, 15, val, pSChan->precision);
		else
		    strcpy(valTxt, "missing");
	    }
	    guiTextFieldPrintf(pPlCtx->PLrestBval_PT,
						"%s %s", valTxt, pSChan->EGU);
	    if (pSChan->pFlags[iE].eof)
		strcpy(valTxt, "EOF");
	    else if (pSChan->pFlags[iE].missing)
		strcpy(valTxt, "missing");
	    else {
		if (sydValAsDbl(pSChan, iE, &val))
		    cvtDblToTxt(valTxt, 15, val, pSChan->precision);
		else
		    strcpy(valTxt, "missing");
	    }
	    guiTextFieldPrintf(pPlCtx->PLrestEval_PT,
						"%s %s", valTxt, pSChan->EGU);
	    for (pSlave=pPlCtx->PLmaster.pHead; pSlave!=NULL;
						pSlave=pSlave->pNext) {
		if (pSlave->pSChan == pSChan)
		    break;
	    }
	}
    }

/*-----------------------------------------------------------------------------
*	draw the time `cursors' on the plot and outline the annotation area
*	for the channel which is selected.  (Because alternate calls to
*	sydPlotTimeCursor draw and erase, pSlave is specified only in
*	conjunction with BegTs.)
*----------------------------------------------------------------------------*/
    first = pSspec->restrictFirstData;
    last = pSspec->restrictLastData;
    range = pSspec->pDeltaSec[last] - pSspec->pDeltaSec[first];
    deltaB = ( (double)valB / 1000. ) * range + pSspec->pDeltaSec[first];
    TsAddDouble(&tsB, &pSspec->refTs, deltaB);
    tsStampToText(&tsB, TS_TEXT_MMDDYY, textB);
    guiTextFieldPrintf(pPlCtx->PLrestB_PT, "%s", textB);
    if (pPlCtx->PLrestBegTs.secPastEpoch != 0) {
	sydPlotTimeCursor(&pPlCtx->PLmaster, &pPlCtx->PLrestBegTs,
						    pPlCtx->pRestrictSlave);
    }
    sydPlotTimeCursor(&pPlCtx->PLmaster, &tsB, pSlave);
    pPlCtx->PLrestBegTs = tsB;
    deltaE = ( (double)valE / 1000. ) * range + pSspec->pDeltaSec[first];
    TsAddDouble(&tsE, &pSspec->refTs, deltaE);
    tsStampToText(&tsE, TS_TEXT_MMDDYY, textE);
    guiTextFieldPrintf(pPlCtx->PLrestE_PT, "%s", textE);
    if (pPlCtx->PLrestEndTs.secPastEpoch != 0) {
	sydPlotTimeCursor(&pPlCtx->PLmaster, &pPlCtx->PLrestEndTs, NULL);
    }
    sydPlotTimeCursor(&pPlCtx->PLmaster, &tsE, NULL);
    pPlCtx->PLrestEndTs = tsE;
    pPlCtx->pRestrictSlave = pSlave;
}

/*+/subr**********************************************************************
* NAME	PLrestrictApply_pb
*
*-*/
void
PLrestrictApply_pb(item)
Panel_item item;
{   PL_CTX	*pPlCtx=(PL_CTX *)xv_get(item, XV_KEY_DATA, PLCTX);
    SYD_SPEC	*pSspec=*pPlCtx->ppSspec;
    TS_STAMP	tsB, tsE;	/* time stamp at begin and end sliders */
    char	textB[28], textE[28];/* text date/time at begin and end */
    char	*pTextB=textB, *pTextE=textE;

    if (pSspec == NULL || pSspec->chanCount <= 0) {
	guiNotice(pPlCtx->pGuiCtx, "no channels selected");
	return;
    }

    strcpy(textB, (char *)xv_get(pPlCtx->PLrestB_PT, PANEL_VALUE));
    tsTextToStamp(&tsB, &pTextB);
    strcpy(textE, (char *)xv_get(pPlCtx->PLrestE_PT, PANEL_VALUE));
    tsTextToStamp(&tsE, &pTextE);

    if (pPlCtx->pRestrictBegTs != NULL)
	*pPlCtx->pRestrictBegTs = tsB;
    if (pPlCtx->pRestrictEndTs != NULL)
	*pPlCtx->pRestrictEndTs = tsE;
    if (pPlCtx->pRestrict_PC != NULL)
	xv_set(*pPlCtx->pRestrict_PC, PANEL_VALUE, 1, NULL);
    sydSampleSetRestrict(*pPlCtx->ppSspec, tsB, tsE);
    PLwinPlot_xvo(item);
}
/*+/subr**********************************************************************
* NAME	PLrestrictReset_pb
*
*-*/
void
PLrestrictReset_pb(item)
Panel_item item;
{   PL_CTX	*pPlCtx=(PL_CTX *)xv_get(item, XV_KEY_DATA, PLCTX);
    SYD_SPEC	*pSspec=*pPlCtx->ppSspec;

    if (pSspec == NULL || pSspec->chanCount <= 0) {
	guiNotice(pPlCtx->pGuiCtx, "no channels selected");
	return;
    }
    if (pPlCtx->pRestrict_PC != NULL)
	xv_set(*pPlCtx->pRestrict_PC, PANEL_VALUE, 0, NULL);
    sydSampleSetRestrict(*pPlCtx->ppSspec, NULL, NULL);
    PLwinPlot_xvo(item);
}

/*+/subr**********************************************************************
* NAME	PLrestrict_init
*
*-*/
#if 0
PLrestrict_init(pPlCtx)
PL_CTX	*pPlCtx;
{
    if ((int)xv_get(item, PANEL_VALUE) == 0) {
	pCtx->pPlCtx->restrictTime = 0;
	if (pCtx->pSspec != NULL)
	    sydSampleSetRestrict(pCtx->pSspec, NULL, NULL);
    }
    else {
	pCtx->pPlCtx->restrictTime = 1;
    }
    pPlCtx->PLrestBegTs.secPastEpoch = 0;
    pPlCtx->PLrestEndTs.secPastEpoch = 0;
    
}
#endif

/*+/subr**********************************************************************
* NAME	PLwinInit - initialize the frame for plotting
*
*-*/
static void
PLwinInit(pPlCtx, title)
PL_CTX	*pPlCtx;
char	*title;
{
    Panel	panel;
    Menu	menu;
    int	y=5, x=5, ht=0;

    pPlCtx->PLwin_CF=guiFrameCmd(title,pPlCtx->pGuiCtx->baseFrame,&panel,NULL);
/*-----------------------------------------------------------------------------
* Plot v  setup...
*   to screen
*   to printer
*   printer setup...
*   to EPS file
*----------------------------------------------------------------------------*/
    menu = guiMenu(NULL, 0,NULL,0,NULL);
    guiMenuItem("to screen",menu,PLwinPlot_mi,0,1,PLCTX,pPlCtx,PLTYPE,PL_PLAIN);
    guiMenuItem("to printer", menu,PLwinPlot_mi,0,0,PLCTX,pPlCtx,PLTYPE,PL_HC);
    guiMenuItem("printer setup...", menu, guiPrinterShowCF_mi, 0, 0,
		GUI_PPRT, pPlCtx->pGuiPrt, 0, NULL);
    guiMenuItem("to EPS file", menu, PLwinPlot_mi, 0, 0,
		PLCTX, pPlCtx, PLTYPE, PL_EPS);
    guiButton("Plot", panel, &x, &y, &ht, NULL, menu, 0, NULL, 0, NULL);
    guiButton("setup...", panel, &x, &y, &ht, guiCFshowPin_xvo, NULL,
		GUI_PCTX, pPlCtx->pGuiCtx, GUI_CF, pPlCtx->PLsetup_CF);
    y += ht + 5; x = 5; ht = 0;

    pPlCtx->PLmaster.pHead = NULL;
    pPlCtx->PLmaster.pTail = NULL;
    pPlCtx->PLmaster.height = 512;
    pPlCtx->PLmaster.width = 512;
    pPlCtx->PLwinCanvas = guiCanvas(pPlCtx->PLwin_CF, x,y, PLwinEvent,
		&pPlCtx->PLwinBg, &pPlCtx->PLwinFg, PLCTX,pPlCtx,0,NULL);
    xv_set(pPlCtx->PLwin_CF, XV_HEIGHT, 512 + y, NULL);
}

/*+/subr**********************************************************************
* NAME	PLinitButton - initialize plot panel, tied to a button
*
* DESCRIPTION
*	Sets up a plot panel and associated objects, including:
*	o   PLOT button, to initiate a plot
*
*	Also initializes the actual plotting frame.
*
* RETURNS
*	void
*
* BUGS
* o	text
*
*-*/
void
PLinitButton(ppPlCtx,pGuiCtx,pGuiPrt,title,panel,pX,pY,pHt,ppSspec)
PL_CTX	**ppPlCtx;
GUI_CTX	*pGuiCtx;
GUI_PRT	*pGuiPrt;
char	*title;
Panel	panel;
int	*pX, *pY, *pHt;
SYD_SPEC **ppSspec;
{
    PL_CTX *pPlCtx;

    if ((pPlCtx = (PL_CTX *)malloc(sizeof(PL_CTX))) == NULL)
	assertAlways(0);
    *ppPlCtx = pPlCtx;
    pPlCtx->pGuiCtx = pGuiCtx;
    pPlCtx->pGuiPrt = pGuiPrt;
    pPlCtx->ppSspec = ppSspec;
    pPlCtx->HCfName[0] = '\0';
    pPlCtx->PLplotting = 0;
    pPlCtx->PLrestBegTs.secPastEpoch = 0;
    pPlCtx->PLrestEndTs.secPastEpoch = 0;
    pPlCtx->useStats = 0;
    pPlCtx->pRestrictBegTs = pPlCtx->pRestrictEndTs = NULL;
    pPlCtx->pRestrict_PC = NULL;
    PLsetupInitCF(pPlCtx, title);
    PLwinInit(pPlCtx, title);
    guiButton("plotting...", panel, pX, pY, pHt, PLwinPlot_xvo, NULL,
			PLCTX, pPlCtx, PLTYPE, PL_NULL);
}
/*+/subr**********************************************************************
* NAME	PLinitMenu - initialize plot panel, tied to a menu item
*
* DESCRIPTION
*	Sets up a plot panel and associated objects, including:
*	o   PLOT menu item, to initiate a plot
*
*	Also initializes the actual plotting frame.
*
* RETURNS
*	void
*
* BUGS
* o	text
*
*-*/
void
PLinitMenu(ppPlCtx, pGuiCtx, pGuiPrt, title, menu, dflt, ppSspec)
PL_CTX	**ppPlCtx;
GUI_CTX	*pGuiCtx;
GUI_PRT	*pGuiPrt;
char	*title;
Menu	menu;
int	dflt;
SYD_SPEC **ppSspec;
{
    PL_CTX *pPlCtx;

    if ((pPlCtx = (PL_CTX *)malloc(sizeof(PL_CTX))) == NULL)
	assertAlways(0);
    *ppPlCtx = pPlCtx;
    pPlCtx->pGuiCtx = pGuiCtx;
    pPlCtx->pGuiPrt = pGuiPrt;
    pPlCtx->ppSspec = ppSspec;
    pPlCtx->HCfName[0] = '\0';
    pPlCtx->PLplotting = 0;
    pPlCtx->useStats = 0;
    pPlCtx->pRestrictBegTs = pPlCtx->pRestrictEndTs = NULL;
    pPlCtx->pRestrict_PC = NULL;

    PLsetupInitCF(pPlCtx, title);
    PLwinInit(pPlCtx, title);
    guiMenuItem("plotting...", menu, PLwinPlot_mi, 0, dflt,
			PLCTX, pPlCtx, PLTYPE, PL_PLAIN);
}

/*+/subr**********************************************************************
* NAME	PLquit
*
*-*/
void
PLquit(pPlCtx, destroyFlag)
PL_CTX	*pPlCtx;
int	destroyFlag;
{
    if (destroyFlag) {
	xv_destroy_safe(pPlCtx->PLsetup_CF);
	xv_destroy_safe(pPlCtx->PLwin_CF);
    }
    unlink(pPlCtx->HCfName);
    free(pPlCtx);
}
