/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
#ifndef INC_PL_h
#define INC_PL_h
/*	@(#)PL.h	1.4 2/23/93
 *	Author:	Roger A. Cole
 *	Date:	02-08-91
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 */
/*+/mod***********************************************************************
* TITLE	PL.h - definitions for PL, the printing software
*
* DESCRIPTION
*
*-***************************************************************************/

typedef enum {
    PL_NULL,	/* no output */
    PL_PLAIN,	/* ordinary print to the screen */
    PL_EPS,	/* output to Encapsulated PostScript file */
    PL_HC,	/* print to printer */
} PL_TYPE;

typedef struct plCtx {
    GUI_CTX	*pGuiCtx;	/* pointer to gui context */
    GUI_PRT	*pGuiPrt;	/* pointer to gui printer context */
    Menu_item	plotHC_MI;	/* menu item for plotHC */
    Menu_item	plotEPS_MI;	/* menu item for plotEPS */
    int		PLplotting;	/* plotting on screen is in progress */
    Panel_item	PLoverlay_PCS;	/* overlay type--grid */
    Panel_item	PLbars_PCS;	/* error bar type--none or stdDev */
    Panel_item	PLhoriz_PCS;	/* horiz axis type--time, value, bin */
    Panel_item	PLscale_PCS;	/* axis auto scaling--no, yes, or shared */
    Panel_item	PLseparate_PCS;	/* separate or combined */
    Panel_item	PLstat_PCS;	/* value status--suppress or show */
    Panel_item	PLwith_PCS;	/* plot with--lines, marks, or points */
    Panel_item	PLrestApply_PB;	/* restricted time, `apply' button */
    Panel_item	PLrestReset_PB;	/* restricted time, `reset' button */
    Panel_item	PLrestB_PSL;	/* restricted time begin, 0 to 1000 */
    Panel_item	PLrestB_PT;	/* restricted time begin, date/time text */
    Panel_item	PLrestBval_PT;	/* restricted time begin channel value */
    TS_STAMP	PLrestBegTs;	/* prior value for begin cursor */
    TS_STAMP	*pRestrictBegTs;/* pointer to saved restricted begin */
    Panel_item	PLrestE_PSL;	/* restricted time end, 0 to 1000 */
    Panel_item	PLrestE_PT;	/* restricted time end, date/time text */
    Panel_item	PLrestEval_PT;	/* restricted time end channel value */
    TS_STAMP	PLrestEndTs;	/* prior value for end cursor */
    TS_STAMP	*pRestrictEndTs;/* pointer to saved restricted end */
    SYD_PL_SLAVE *pRestrictSlave;/* pointer to plot slave for channel */
    Panel_item	PLrestChan_PT;	/* restricted time channel, name */
    Panel_item	dir_PT;		/* PANEL_TEXT item for EPS directory */
    char	dir[GUI_TDIM];	/* EPS directory */
    Panel_item	file_PT;	/* PANEL_TEXT item for EPS file name */
    char	file[GUI_TDIM];	/* EPS file name */
    Panel_item	fileMode_PCS;	/* PANEL_CHOICE_STACK item for file mode */
    Frame	PLsetup_CF;	/* command frame for setting up plots */
    Frame	PLwin_CF;	/* command frame containing plot canvas */
    Canvas	PLwinCanvas;	/* the plot canvas */
    unsigned long PLwinBg;	/* background pixel value */
    unsigned long PLwinFg;	/* foreground pixel value */
    int		PLwinWidth;	/* width of plot window frame */
    int		PLwinHeight;	/* height of plot window frame */
    PPR_WIN	*pWin;		/* pprPlot window structure pointer */
    char	HCfName[GUI_TDIM];/* temp file for printing plots */
    SYD_SPEC	**ppSspec;	/* pointer to pointer to sample set spec */
    short	useStats;	/* 1 if to plot mean,stdDev */
    short	useMeans;	/* 1 if to plot mean */
    Panel_item	*pRestrict_PC;	/* pointer to panel choice for `restrict' */
    Panel_item	stripDelay_PT;	/* PANEL_TEXT item for strip chart delay */
    float	stripDelay;	/* strip chart delay, in seconds */
    TS_STAMP	stripLastTs;	/* strip chart time of last shift */
    Panel_item	stripMode_PCS;	/* PANEL_CHOICE STACK item for strip mode */
    SYD_PL_MSTR	PLmaster;	/* sync data plot master struct */
} PL_CTX;

void PLinitButton(), PLinitMenu(), PLquit();

#endif	/* INC_PL_h */
