/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
#ifndef INC_arAccessLib_h
#define INC_arAccessLib_h
/*	arAccessLib.h,v 1.2 2002/08/02 15:37:52 jba Exp
 *	Author:	Roger A. Cole
 *	Date:	03-09-90
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 * make options
 *	-DvxWorks	makes a version for VxWorks
 *	-DNDEBUG	don't compile assert() checking
 *      -DDEBUG         compile various debug code, including checks on
 *                      malloc'd memory
 */
/*+/mod***********************************************************************
* TITLE	arAccessLib.h - AR `data set access library' definitions
*
* DESCRIPTION
*	
*-***************************************************************************/

#ifndef INC_arChanIO_h
#   include <arChanIO.h>
#endif
#ifndef INC_genDefs_h
#   include <genDefs.h>
#endif
#ifndef INC_tsDefs_h
#   include <tsDefs.h>
#endif
#ifndef INC_bfDefs_h
#   include <bfDefs.h>
#endif
#ifndef INCLcadefh
#   include <cadef.h>
#endif
#ifndef INCerrMdefh
#   include <errMdef.h>	/* get M_arAcc for this subsystem's `number' */
#endif


/*----------------------------------------------------------------------------
* channel data file--organization
*
*    The AR channel data file supports storing and retrieving data for
*    GTACS channels.  Within the file, each channel exists as a sub-file
*    which can be opened, closed, read, written, etc.
*
*    The objective for each sub-file is to store enough information to
*    properly respond to requests for historical data with request types
*    of DBR_xxx, DBR_STS_xxx, DBR_TIME_xxx, and DBR_GR_xxx.
*
*---------------------------------------------------------------------------*/

/*/subhead status codes-------------------------------------------------------
*		S T A T U S   C O D E S
*
*	
*    Status codes of the form S_ts_briefMessage are used with the AR access
*    library routines.
*
*    The macro ArAccStatusToText(stat) can be used to obtain a text string
*    corresponding to a status code.
*
*----------------------------------------------------------------------------*/
#define S_arAcc_OK		(M_arAcc|0| 0<<1) /* success */
#define S_arAcc_noMem		(M_arAcc|1| 1<<1) /* couldn't malloc memory */
#define S_arAcc_chanNotFound	(M_arAcc|1| 2<<1) /* couldn't find channel */
#define S_arAcc_ERROR		(M_arAcc|1| 3<<1) /* other error */

#define ARACC_S_PAST		 4	/* one past last legal code */

#define ArAccStatusToIndex(status) \
	(  ((status&0xffff0000)!=M_arAcc)  \
		? ARACC_S_PAST  \
		: (  (((status&0xffff)>>1)>=ARACC_S_PAST)  \
		     ? ARACC_S_PAST  \
		     : ((status&0xffff)>>1)  \
		  )  \
	)

#define ArAccStatusToText(status) \
	(glArAccStatText[ArAccStatusToIndex(status)])

#ifndef ARACC_TEXT_GLBLSOURCE
    extern char *glArAccStatText[];
#else
    char *glArAccStatText[] = {
	/* S_arAcc_OK            */ "success",
	/* S_arAcc_noMem         */ "couldn't malloc memory",
	/* S_arAcc_chanNotFound  */ "couldn't find channel",
	/* S_arAcc_ERROR         */ "error",

	/* ARACC_S_PAST          */ "illegal arAcc status code",
    };
#endif

/*/subhead AR_CF_DESC---------------------------------------------------------
* AR_CF_DESC - AR channel data file descriptor
*
*    This structure is the 'root' of all information for an open AR channel
*    data file, including:
*    o	the interface to the block file routines on which the channel file
*	is implemented
*    o	a list of the AR channel descriptors which are currently open
*    o  a list of all the master index buffers in memory
*    o  pointers to all the index and data buffers in memory (this
*	information requires looking in the various channel descriptors)
*
*    In most cases, using this structure is straightforward.  One real ugly
*    exception is the 'free chanHdr list', based on pMIFree and MIFree_hdrNum.
*    These two items lead directly to the first free chanHdr, but finding the
*    NEXT free chanHdr is less direct and may even require a search of the
*    MI blocks in memory.  (See arCF_MIAddChan() for more details.)
*---------------------------------------------------------------------------*/
typedef struct {
    BF_DESC	*pBfDesc;	/* pointer to block file descriptor */
    short	b0Modified;	/* block 0 has been modified */
    void	*pWriteHead;	/* ptr to head of "write list" */
    void	*pWriteTail;	/* ptr to tail of "write list" */
    AR_MI_BUF	*pMIHead;	/* pointer to buf for first MI block */
    AR_MI_BUF	*pMITail;	/* pointer to buf for last MI block */
    AR_MI_BUF	*pMIFree;	/* pointer to buf containing first MIFree */
    short	MIFree_hdrNum;	/* chanHdr subscript in *pMIFree */
    struct arChanDesc *pChanDescHead;	/* ptr to head of chanDesc list */
    struct arChanDesc *pChanDescTail;	/* ptr to tail of chanDesc list */
} AR_CF_DESC;

/*/subhead AR_CHAN_DESC-------------------------------------------------------
* AR_CHAN_DESC - AR channel descriptor for accessing a channel in an AR
*	channel data file
*
*    This channel descriptor allows treating a channel as a file, with
*    create, open, read, write, purge, and delete operations provided.  The
*    read and write operations are macro-based, so that subroutine calls are
*    necessary only when it is necessary to switch blocks.
*
*    The handling of remainCount deserves special understanding.  It is
*    initialized to -10 when the chanDesc is first opened.  This is used as
*    a cue when reading that the first block is being read.  For the first
*    block, any overflow bytes are skipped.
*
*    datInfNum is also initialized to -10 when the chanDesc is first opened.
*    This is done to force the reading of the first index block for the
*    channel.
*
*    The AR_CDESC_SUP flag is used to allow more efficient processing for
*    transferring data.  Typical usage is to reset this flag before
*    transferring a stream of bytes, transfer the bytes without checking
*    status, and then act on the status for the last byte transferred.
*    Routines which are involved in the transfer set this flag after the
*    first error message has been printed, suppressing additional messages.
*
*    Some convenience macros are supplied for dealing with this structure.
*    These macros use information from within the chanHdr.
*
*	ArCDChanHdr(pChanDesc)	accesses the chanHdr corresponding to chanDesc
*				pChanHdr = &ArCDChanHdr(pChanDesc)
*				chanName = ArCDChanHdr(pChanDesc).name
*	ArCDDatInfo(pChanDesc)	accesses the datInfo corresponding to chanDesc
*				pDatInfo = &ArCDDatInfo(pChanDesc)
*				firstSec = ArCDDatInfo(pChanDesc).secPastEpoch
*	ArChanElementCount(pChanDesc)	returns native count for channel
*	ArChanFieldType(pChanDesc)	returns native type for channel
*	ArChanName(pChanDesc)		returns name of channel
*	ArChanNStates(pChanDesc)	returns # of states for ENUM channel
*	ArChanPrec(pChanDesc)		returns digits of precision for channel
*	ArChanStates(pChanDesc)		returns state strings for channel
*	ArChanUnits(pChanDesc)		returns units text for channel
*---------------------------------------------------------------------------*/
typedef struct arChanDesc {
    struct arChanDesc *pNextDesc;/* forward link to next descriptor */
    struct arChanDesc *pPrevDesc;/* backward link to prev descriptor */
    AR_CF_DESC	*pArCfDesc;	/* ptr to channel file descriptor */
    AR_DATA_BUF *pDataBuf;	/* pointer to data buffer */
    char	*pData;		/* pointer to last character read
				or written */
    AR_INDEX_BUF *pIndexBuf;	/* pointer to buffer with index */
    AR_MI_BUF	*pMIBuf;	/* pointer to buffer with master indx */
    ULONG	flags;		/* flags--combination of AR_CDESC_xxx */
#define		    AR_CDESC_WRITE  1	/* channel opened O_RDWR */
#define		    AR_CDESC_NEW    2	/* channel is new */
#define		    AR_CDESC_SUP    4	/* suppress conditional messages */
#define		    AR_CDESC_EOF    8	/* end of file, on read */
#define		    AR_CDESC_BEGIN 16	/* begin a run of data taking */
#define		    AR_CDESC_INACT 32	/* end a run, due to inactivation */
#define		    AR_CDESC_DISCON 64	/* end a run, due to disconnect */
#define		    AR_CDESC_SNAP_BEGIN 128 /* begin a run--snapshot begin */
#define		    AR_CDESC_SNAP_END 256 /* end a run--snapshot end */
    short	remainCount;	/* number of characters left to read
				in data block or number of available
				character positions left for writing */
    short	datInfNum;	/* index for datInfo in index block */
    short	hdrNum;		/* index for chanHdr in MI block */
    TS_STAMP	timeStamp;	/* time stamp of last data transferred */
} AR_CHAN_DESC;

#define ArCDChanHdr(pChanDesc) pChanDesc->pMIBuf->chanHdr[pChanDesc->hdrNum]
#define ArCDDatInfo(pChanDesc) \
			pChanDesc->pIndexBuf->datInfo[pChanDesc->datInfNum]


/*/subhead convenience macros--------------------------------------------------
* convenience macros
*----------------------------------------------------------------------------*/
#define ArCFChanElementCount(pChanDesc) \
    ((unsigned int)((pChanDesc)->pMIBuf->chanHdr[(pChanDesc)->hdrNum].elCount))
#define ArCFChanFieldType(pChanDesc) \
    ((chtype)((pChanDesc)->pMIBuf->chanHdr[(pChanDesc)->hdrNum].chanType))
#define ArCFChanName(pChanDesc) \
    ((pChanDesc)->pMIBuf->chanHdr[(pChanDesc)->hdrNum].name)
#define ArCFChanNStates(pChanDesc) \
    ((pChanDesc)->pMIBuf->chanHdr[(pChanDesc)->hdrNum].graphics.enumGr.pGRBuf->numStrings)
#define ArCFChanPrec(pChanDesc) \
    ((pChanDesc)->pMIBuf->chanHdr[(pChanDesc)->hdrNum].graphics.floatGr.precision)
#define ArCFChanStates(pChanDesc) \
    ((pChanDesc)->pMIBuf->chanHdr[(pChanDesc)->hdrNum].graphics.enumGr.pGRBuf->states)
#define ArCFChanUnits(pChanDesc) \
    ((pChanDesc)->pMIBuf->chanHdr[(pChanDesc)->hdrNum].graphics.floatGr.units)


/*-----------------------------------------------------------------------------
* prototypes
*----------------------------------------------------------------------------*/
long  arCFChanClose();
long  arCFChanDel();
long  arCFChanFind();
AR_CHAN_DESC *arCFChanOpen();
long  arCFChanPosition();
long  arCFChanPurge();
long  arCFChanRead();
long  arCFChanWrite();
long  arCFChanWrite_args();
long  arCFChanWriteByte();
long  arCFChanWriteGR();

long  arCFClose();
AR_CF_DESC *arCFCreate();
AR_CF_DESC *arCFOpen();
long  arCFSyncWrite();

AR_CF_DESC *arCF_mallocCF_DESC();
AR_CHAN_DESC *arCF_mallocCHAN_DESC();
void arCF_freeCF_DESC();
void arCF_freeCHAN_DESC();

#endif
