/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
#ifndef INC_arDefs_h
#define INC_arDefs_h
/*	@(#)arDefs.h	1.5 1/7/92
 *	Author:	Roger A. Cole
 *	Date:	12-08-89
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 */
/*+/mod***********************************************************************
* TITLE	arDefs.h - definitions for using AR code
*
* DESCRIPTION
*	This file contains various definitions needed by AR code and by
*	programs which use the AR code.
*
*    TYPEDEFS
*
*	AR_REQ  is an archive request
*	AR_METH is archive method information
*	AR_OUT	is archive request file output information
*	AR_EL	is an element in an archive request
*
* SEE ALSO:
* o	db_access.h	contains definitions for database related
*			quantities.
* o	cadef.h		contains some additional symbol definitions
*
*-***************************************************************************/

#include <stdio.h>
#include <setjmp.h>

#ifndef INC_genDefs_h
#   include <genDefs.h>
#endif
#ifndef INC_tsDefs
#   include <tsDefs.h>
#endif
#ifndef INC_arCs_h
#   include <arCS.h>
#endif
#ifndef INC_arAccessLib_h
#   include <arAccessLib.h>
#endif
#ifndef INCLcadefh
#   include <cadef.h>
#endif

/*----------------------------------------------------------------------------
* CONTEXT
*
*    `pglArCtx' is a pointer to a context block for use by the routines in
*    arLib.  A task using these routines must define a context block,
*    initialize it, and put a pointer to the block in `pglArCtx'.
*---------------------------------------------------------------------------*/
extern struct ar_ctx *pglArCtx;		/* pointer to context block */

/*----------------------------------------------------------------------------
* FILE DIRECTORIES and PATHS
*---------------------------------------------------------------------------*/

#define AR_FILE_SET_DIR "arSet/"
#define AR_FILE_SET_TXT_EXT ".txt"
#define AR_FILE_CHAN_PATH "arChan/arChanData.bin"
#define AR_FILE_CHAN_BIN_EXT ".bin"
#define AR_FILE_REQ_DIR "arReq/"
#define AR_FILE_REQ_EXT ".req"
#define AR_FILE_HIS_DIR "arReq/"
#define AR_FILE_HIS_EXT ".his"
#define AR_FILE_ENB_PATH "arReq/enabled.dat"
#define AR_FILE_DIS_PATH "arReq/disabled.dat"

/*/subhead AR_ITEM_xxx--------------------------------------------------------
* ITEM TYPE
*	Structures which contain Channel Access channel pointers begin with
*	an `item type', which is used primarily by the connection handler
*	to distinguish the various kinds of structures.
*
*	Such structures always have item type as their first item, with a
*	pointer to the associated archive request as the second item.
*---------------------------------------------------------------------------*/
#define ArItemToText(type) \
	(glArItemText[(type<0||type>=AR_ITEM_PAST)?AR_ITEM_PAST:type])

#ifndef AR_TEXT_GLBLSOURCE
    extern char *glArItemText[];
#else
    char *glArItemText[] = {
        /* AR_ITEM_UNDEF       */      "undefined",
        /* AR_ITEM_METH_ACT    */      "activation method",
        /* AR_ITEM_METH_INACT  */      "inactivation method",
        /* AR_ITEM_METH_SAMPLE */      "sample method",
        /* AR_ITEM_EL          */      "element",
        /* AR_ITEM_STAT_REQ    */      "request status channel",
        /* AR_ITEM_STAT_SAMP   */      "sample status channel",

        /* AR_ITEM_PAST        */      "illegal"
    };
#endif

enum arItem {
    AR_ITEM_UNDEF,	/* MUST BE ZERO!!! */
    AR_ITEM_METH_ACT,
    AR_ITEM_METH_INACT,
    AR_ITEM_METH_SAMPLE,
    AR_ITEM_EL,
    AR_ITEM_STAT_REQ,
    AR_ITEM_STAT_SAMP,

    AR_ITEM_PAST	/* last legal + 1 */
};

/*/subhead misc meth types----------------------------------------------------
* AFTER METHOD
*	AR_METH_ST_UNARMED -- waiting for req.sampleCount >= afterCount
*	AR_METH_ST_ARMED -- sampleCount has reached or exceeded afterCount
*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
* ALWAYS METHOD
*	AR_METH_ST_ARMED -- perform the method's action
*
*	When the action is complete, the method is removed from the queue.
*	(Thus this method is reasonable only for activation.)
*---------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
* DEMAND METHOD
*	AR_METH_ST_UNARMED -- waiting for a "demand" request
*	AR_METH_ST_ARMED -- a "demand" request has been received
*
*	When the action is complete, the method is moved back to UNARMED.
*---------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
* MONITOR METHOD
*	AR_METH_ST_ARMED -- sampling can begin.  (To actually begin sampling,
*		ca_add_array_event will need to be done for each element in
*		the request.)
*----------------------------------------------------------------------------*/

/*----------------------------------------------------------------------------
* NEVER METHOD
*	AR_METH_ST_UNARMED -- never perform the method's action
*
*	(A method of NEVER type should never appear on a method queue.  This
*	method is reasonable only for inactivation.)
*---------------------------------------------------------------------------*/

/*/subhead event--------------------------------------------------------------
* EVENT METHOD
*	'count' determines how many times the event condition must be
*	satisfied before the event action is taken.
*
*	The possible states for an event method (meth.state) are:
*	AR_METH_ST_NEW -- no values have yet been received from Channel
*		Access.  When the first value is received, move to UNARMED.
*	AR_METH_ST_UNARMED -- waiting for condition to be satisfied
*	AR_METH_ST_ARMED -- condition is satisfied
*	AR_METH_ST_SUCCESS -- the condition has been satisfied 'count' times
*
*	When the event condition is 'change', some special handling of state
*	is done.  Initially, the state is UNARMED.  The first value reported
*	by Channel Access changes the state to ARMED.  Subsequent values
*	reported by Channel Access are treated as satisfying the 'change'
*	condition.
*
*	Initially, 'counter' is assigned the value from 'count'; then
*	'counter' is decremented each time there is a transition to ARMED.
*	When 'counter' reaches zero, the method succeeds and the state is
*	set to SUCCESS.  When action is complete, the state will be set
*	back to UNARMED.
*
*---------------------------------------------------------------------------*/

#define ArEvTypeToText(type) \
			(glArEvKw[(type<0||type>=AR_EV_PAST)?AR_EV_PAST:type])

#ifndef AR_TEXT_GLBLSOURCE
    extern char *glArEvKw[];
#else
    char *glArEvKw[] = {
        /* AR_EV_UNDEF     */   "undefined",
        /* AR_EV_CHANGE    */   "change",
        /* AR_EV_EQ        */   "==",
        /* AR_EV_NEQ       */   "!=",
        /* AR_EV_GT        */   ">",
        /* AR_EV_GE        */   ">=",
        /* AR_EV_LT        */   "<",
        /* AR_EV_LE        */   "<=",
        /* AR_EV_PAST      */   "illegal"
    };
#endif

enum arEvType {
    AR_EV_UNDEF,		/* MUST BE ZERO!!! */
    AR_EV_CHANGE,
    AR_EV_EQ,
    AR_EV_NEQ,
    AR_EV_OK_NON_NUM = AR_EV_NEQ,  /* event types above this are OK for ENUM,
				STRING, and CHAR (i.e., non-numeric) channels */
    AR_EV_GT,
    AR_EV_GE,
    AR_EV_LT,
    AR_EV_LE,

    AR_EV_PAST		/* last legal + 1 */
};

union arMeEvVal {
    short	shrtval;
    float	fltval;
    char	strval[AR_STRVAL_DIM];
};

struct arMeEv {
    enum arEvType type;			/* event type: AR_EV_xxx */
    short	count;			/* count between actions */
    short	counter;		/* current count; act on zero */
    chid	pCh;			/* channel pointer */
    char	name[AR_NAME_DIM];	/* channel name */
    evid	pEv;			/* event pointer from ca_add_event */
    TS_STAMP	timeStamp;		/* time stamp for value from CA */
    union arMeEvVal val;		/* value from Channel Access */
    union arMeEvVal oldVal;		/* previous value from CA */
    union arMeEvVal compVal;		/* comparison value */
};

/*/subhead periodic/timed-----------------------------------------------------
* PERIODIC METHOD
* TIMED METHOD
*	'period' determines the interval between actions.  'nextTime' indicates
*	the next time the action is to be taken.  Both are in the format of
*	GTACS time stamps.
*
*	The condition which must be satisfied is that present time must be
*	greater than 'nextTime'.
*
*	For PERIODIC, handling is slightly different for activation methods
*	and sample methods.  For activation methods, when the request is
*	enabled, 'nextTime' is set to present time and state is set to ARMED,
*	so that the first activation will occur immediately.  When the
*	request is activated (both for the first time and also for all
*	subsequent times), 'nextTime' is set to 'nextTime' + 'period'.  (If
*	there has been excessive delay in acting on the activation, the
*	new nextTime might be less than current time; if this occurs,
*	'nextTime' is set to 'currentTime' + 'period'.)  For sample methods,
*	when the request is activated, 'nextTime' is set to present time
*	and state is set to ARMED, so that the first sample will occur
*	immediately; each time the sample is taken, 'nextTime' is set to
*	'currentTime' + 'period'.
*
*	For TIMED, when the request is activated, 'nextTime' is
*	set to present time plus 'period', and state is set to ARMED.  This
*	method only applies to inactivation methods; when the method succeeds,
*	the request is inactivated.
*---------------------------------------------------------------------------*/
struct arMePer {
    TS_STAMP	period;		/* period length, as time stamp */
    TS_STAMP	nextTime;	/* time to perform action, time stamp */
};

/*/subhead AR_METH------------------------------------------------------------
*    generic method-related items
*----------------------------------------------------------------------------*/
#define ArMethTypeToText(type) \
	(glArMethKw[(type<0||type>=AR_METH_PAST)?AR_METH_PAST:type])

#ifndef AR_TEXT_GLBLSOURCE
    extern char *glArMethKw[];
#else
    char *glArMethKw[] = {
        /* AR_METH_UNDEF     */ "undefined",
        /* AR_METH_DEMAND    */ "demand",
        /* AR_METH_EVENT     */ "event",
        /* AR_METH_PERIODIC  */ "periodic",
        /* AR_METH_SCHED     */ "scheduled",
        /* AR_METH_ALWAYS    */ "always",
        /* AR_METH_NEVER     */ "never",
        /* AR_METH_AFTER     */ "after",
        /* AR_METH_MONITOR   */ "monitor",
        /* AR_METH_TIMED     */ "timed",
        /* AR_METH_PAST      */ "illegal"
    };
#endif

enum arMeType {
    AR_METH_UNDEF,	/* MUST BE ZERO!!! */
    AR_METH_DEMAND,
    AR_METH_EVENT,	/* chanName condition ... */
    AR_METH_PERIODIC,	/* hh:mm:ss */
    AR_METH_SCHED,	/* need date stamp and time stamp */
    AR_METH_ALWAYS,
    AR_METH_NEVER,
    AR_METH_AFTER,	/* need number of samples */
    AR_METH_MONITOR,
    AR_METH_TIMED,	/* hh:mm:ss */

    AR_METH_PAST	/* last legal + 1 */
};

#define ArMethStateToText(state) \
    (glArMeStateText[(state<0||state>=AR_METH_ST_PAST)?AR_METH_ST_PAST:state])

#ifndef AR_TEXT_GLBLSOURCE
    extern char *glArMeStateText[];
#else
    char *glArMeStateText[] = {
        /* AR_METH_ST_UNDEF     */      "undefined",
        /* AR_METH_ST_NEW       */      "needs init",
        /* AR_METH_ST_UNARMED   */      "unarmed",
        /* AR_METH_ST_ARMED     */      "armed",
        /* AR_METH_ST_SUCCESS   */      "action pending",
        /* AR_METH_ST_ACTION    */      "action in progress",
        /* AR_METH_ST_COMPLETE  */      "action complete",
        /* AR_METH_ST_PAST      */      "illegal"
    };
#endif

enum arMeState {
    AR_METH_ST_UNDEF,	/* MUST BE ZERO!!! */
    AR_METH_ST_NEW,
    AR_METH_ST_UNARMED,
    AR_METH_ST_ARMED,
    AR_METH_ST_SUCCESS,
    AR_METH_ST_ACTION,
    AR_METH_ST_COMPLETE,

    AR_METH_ST_PAST	/* last legal + 1 */
};

#define ArMethActionToText(type) \
	(glArMeActionText[(type<0||type>=AR_METH_AC_PAST)?AR_METH_AC_PAST:type])

#ifndef AR_TEXT_GLBLSOURCE
    extern char *glArMeActionText[];
#else
    char *glArMeActionText[] = {
        /* AR_METH_AC_UNDEF     */      "undefined",
        /* AR_METH_AC_ACT       */      "activate",
        /* AR_METH_AC_INACT     */      "inactivate",
        /* AR_METH_AC_SAMPLE    */      "sample",

        /* AR_METH_AC_PAST      */      "illegal"
    };
#endif

enum arMeAct {
    AR_METH_AC_UNDEF,	/* MUST BE ZERO!!! */
    AR_METH_AC_ACT,
    AR_METH_AC_INACT,
    AR_METH_AC_SAMPLE,

    AR_METH_AC_PAST	/* last legal + 1 */
};

/*----------------------------------------------------------------------------
* AR_METH
*
* DESCRIPTION
*	A method determines when an action is to occur.  The method structure,
*	defined below, specifies the information to be checked.  A method
*	function (i.e., the code) performs the checks.  There are a variety of
*	method functions corresponding to the variety of method types.
*
*	The essence of a method is to specify a condition to be satisfied and
*	an action to be taken when the condition is satisfied.  The different
*	methods provide a variety of ways to specify conditions, some very
*	simple, and some complex.
*
*	A method is an item in a request.  In actual practice, a request
*	will contain several methods, each serving a different purpose.  The
*	function arReqRead reads a request specification from a file and
*	places the information in the request structure; part of this process
*	is to fill in the method structures within the request.  arReqRead
*	does validation checking for the methods individually, and for some
*	restrictions on how the methods interact with each other.  arReqRead
*	also does some generic initialization of the methods.
*
*	In order to bring a method into use--so that it can actually do
*	something--it is added to a method queue.  When a method is added
*	to a queue, detailed initialization is done.  As an example, when
*	an event method is added to a queue, monitoring is begun for the
*	method's channel.  There are a variety of method queues corresponding
*	to the variety of methods.
*
*	A method will be on a method queue only if the request containing
*	the method is on the main request list.  When a request is removed
*	from the main request list (usually because one of its channels
*	has been disconnected), the request's methods are removed from the
*	method queues, as necessary.
*
* BUGS
* o	SCHED method aren't currently supported
*---------------------------------------------------------------------------*/
typedef struct arMe {
    enum arItem	itemType;	/* type of item in request: AR_ITEM_METH_xxx */
    struct arReq *pReq;		/* ptr to req containing this method */
    enum arMeType type;		/* type of method: AR_METH_xxx */
    enum arMeState state;	/* state of method: AR_METH_ST_xxx */
    short	demand;		/* "demand" from operator */
    enum arMeAct actionType;	/* type of action: AR_METH_AC_xxx */
    struct arMe *pNext;		/* next method in queue */
    struct arMe *pPrev;		/* previous method in queue */
    TS_STAMP	timeStamp;	/* time that condition was last satisfied */
    union {
        struct arMeEv event;	/* event method, control information */
	struct arMePer per;	/* periodic method, control information */
	long	afterCount;	/* after method; sample count to succeed */
    } ctrl;
} AR_METH;			/* method information */


/*/subhead OUTPUT-------------------------------------------------------------
* ARCHIVE OUTPUT related declarations
*
*    Output activities for an archive request are done only if the
*    request status is AR_REQ_ACTIVE.  For any other status, any output
*    activities which may be pending or in progress are not done.
*
*   SAMPLE SET output
*	When the need for a sample is determined (generally by the request's
*	sample method), the output status is set to AR_OUTS_SAMPLE.  If
*	the output status isn't AR_OUTS_IDLE, then the sample method will
*	print an error message and discard the sample request.
*
*	When the sample routine notices a request with AR_OUTS_SAMPLE, it
*	issues ca_get_callback calls for all the elements in the request
*	and sets the output status to AR_OUTS_GETPEND.  (Queueing the
*	gets is done outside the sample method since the callback handler
*	will be called immediately for local channels.  The amount of
*	processing which might result is undesirable to do in the method
*	context.)
*
*	When the gets are all complete, the output status will be changed
*	to AR_OUTS_GETDONE.
*
*	When the sample set write task notices a request with AR_OUTS_GETDONE,
*	it begins the process of actually writing the sample set data to
*	disk.  First, it sets the output status to AR_OUTS_FILING.  Then
*	it opens the file, writes the data, and closes the file.  (If the
*	data are to be written to two places, both are opened and the
*	writes occur in parallel.)  When processing is complete, the output
*	status is set to AR_OUTS_IDLE.
*
*    CHANNEL DATA output
*	When arReqConnMake processes a request with chan:bin output, in
*	addition to making channel access connections, it also opens a
*	write mode chanDesc for each element.  (If the channel data file
*	isn't open, arReqConnMake opens it in write mode.)  If there is
*	presently no stream for a channel, the stream is created and
*	graphics information is obtained and stored in the file.  The
*	chanDesc's are left open until the request is disabled.
*
*	When a request is activated, Channel Access monitors are started
*	for each element.  The handler for the monitors is the
*	arCFChanWrite routine, which sends data `directly' to the file.
*	The output status is set to AR_OUTS_MONITOR.
*
*	When a request is inactivated, the Channel Access monitors are
*	cleared and the output status is set to AR_OUTS_IDLE.
*
* BUGS
*---------------------------------------------------------------------------*/

#define ArOutStatToText(type) \
	(glArOutStatText[(type<0||type>=AR_OUTS_PAST)?AR_OUTS_PAST:type])

#ifndef AR_TEXT_GLBLSOURCE
    extern char *glArOutStatText[];
#else
    char *glArOutStatText[] = {
      /* AR_OUTS_IDLE       */   "idle",
      /* AR_OUTS_SAMPLE     */   "sample needed",
      /* AR_OUTS_GETPEND    */   "sample in progress",
      /* AR_OUTS_GETDONE    */   "sample complete",
      /* AR_OUTS_MONITOR    */   "sampling on change",
      /* AR_OUTS_FILING     */   "file i/o in progress",
      /* AR_OUTS_PAST       */   "illegal"
    };
#endif

enum arOutStat {
    AR_OUTS_IDLE,	/* MUST BE ZERO!!! */
    AR_OUTS_SAMPLE,
    AR_OUTS_GETPEND,
    AR_OUTS_GETDONE,
    AR_OUTS_MONITOR,
    AR_OUTS_FILING,

    AR_OUTS_PAST		/* last legal + 1 */
};

#define ArOutTypeToText(type) \
	(glArOutTypeText[(type<0||type>=AR_OUT_PAST)?AR_OUT_PAST:type])

#ifndef AR_TEXT_GLBLSOURCE
    extern char *glArOutTypeText[];
#else
    char *glArOutTypeText[] = {
      /* AR_OUT_UNDEF      */   "undefined",
      /* AR_OUT_NONE       */   "none",
      /* AR_OUT_SET_TXT    */   "set:txt",
      /* AR_OUT_CHAN_BIN   */   "chan:bin",
      /* AR_OUT_PAST       */   "illegal"
    };
#endif

enum arOutType {
    AR_OUT_UNDEF,	/* MUST BE ZERO!!! */
    AR_OUT_NONE,
    AR_OUT_SET_TXT,
    AR_OUT_CHAN_BIN,

    AR_OUT_PAST		/* last legal + 1 */
};

typedef struct arOut {
    enum arOutStat status;	/* status of output operations AR_OUTS_xxx */
    enum arOutType ARType;	/* type of output to AR base AR_OUT_xxx */
    AR_CF_DESC	*pARCFDesc;	/* pointer to chanFile desc for AR file */
    enum arOutType usrType;	/* type of output to user base AR_OUT_xxx */
    AR_CF_DESC	*pUsrCFDesc;	/* pointer to chanFile desc for user file */
    char	usrPath[GEN_FNAME_DIM]; /* path to user base */
} AR_OUT;

/*/subhead AR_EL--------------------------------------------------------------
* AR_EL	  archive element related declarations
*
*	An archive element stores information and data for a channel.  An
*	archive request typically has one or more archive elements; the
*	elements are linked together as a doubly threaded list, with
*	head and tail pointers in the request.
*
*	The `data buffer' in an element is actually a union of dbr_time_xxx
*	structures.  (No other structures are required in the union, since
*	acquiring data from Channel Access always uses DBR_TIME_xxx types.)
*	For array channels, the size of the element is increased to allow
*	additional values at the end of the dbr_time_xxx structure.
*
*	An element begins life with a call to arElCreateAndAppend, which
*	creates the element and appends it to the end of the element list
*	for the request, and does some limited initialization.  The status
*	for the element is set to AR_EL_NOTCONN.
*
*	When the ca_search is done for an element, its status is changed
*	to AR_EL_CONNPEND.  (An element which was previously connected, but
*	which has received a disconnect notification from Channel Access,
*	also has its status changed to AR_EL_CONNPEND; this is valid, since
*	Channel Access does an automatic re-connect in such cases.)
*
*	When the connection is successful, the element status is changed to
*	AR_EL_CONNOK.  At this time, array channels are detected.  If the
*	channel actually is an array, then an element large enough to hold
*	the array is allocated and linked into the element list in place of
*	the original element.
*
*	When requests are made to obtain data from Channel Access, the
*	element status is changed to either AR_EL_MONPEND or AR_EL_GETPEND,
*	as appropriated.  When a value is received in response to the request,
*	the status is changed to either AR_EL_MONITOR or AR_EL_GETDONE.
*
*---------------------------------------------------------------------------*/

#define ArElStatusToText(status) \
	(glArElStatText[(status<0||status>=AR_EL_PAST)?AR_EL_PAST:status])

#ifndef AR_TEXT_GLBLSOURCE 
    extern char *glArElStatText[]; 
#else 
    char *glArElStatText[] = {
     /* AR_EL_UNDEF     */    "undefined",
     /* AR_EL_NOTCONN   */    "not connected",
     /* AR_EL_CONNPEND  */    "connection pending",
     /* AR_EL_CONNOK    */    "connected",
     /* AR_EL_MONPEND   */    "monitor pending",
     /* AR_EL_MONITOR   */    "being monitored",
     /* AR_EL_PUTPEND   */    "ca_put pending",
     /* AR_EL_PUTFAIL   */    "ca_put failed",
     /* AR_EL_GETPEND   */    "ca_get pending",
     /* AR_EL_GETFAIL   */    "ca_get failed",
     /* AR_EL_GETDONE   */    "ca_get complete",
     /* AR_EL_PAST      */    "illegal"
    };
#endif

enum arElStat {
    AR_EL_UNDEF,		/* MUST BE ZERO !!! */
    AR_EL_NOTCONN,
    AR_EL_CONNPEND,
    AR_EL_CONNOK,
    AR_EL_MONPEND,
    AR_EL_MONITOR,
    AR_EL_PUTPEND,
    AR_EL_PUTFAIL,
    AR_EL_GETPEND,
    AR_EL_GETFAIL,
    AR_EL_GETDONE,

    AR_EL_PAST		/* last legal + 1 */

};

typedef struct arEl {
    enum arItem	itemType;	/* type of item in request: AR_ITEM_EL */
    struct arReq *pReq;		/* pointer to request containing this el */
    struct arEl *pNext;		/* pointer to next, or NULL */
    struct arEl *pPrev;		/* pointer to previous, or NULL */
    chid	pCh;		/* channel pointer */
    evid	pEv;		/* event pointer from ca_add_event */
    char	name[AR_NAME_DIM];/* channel name */
    AR_CHAN_DESC *pARChanDesc;	/* pointer to channel descriptor (used for
				'by channel' storing of data to AR file) */
    AR_CHAN_DESC *pUsrChanDesc;/* pointer to channel descriptor (used for
				'by channel' storing of data to user file) */
    enum arElStat status;	/* status of archive element: AR_EL_xxx */
    chtype	dbrType;	/* DBR_xxx type of value in buffer */
    long	sampleCount;	/* number of samples this sample episode */
    union {
	struct dbr_time_string tstrval;
	struct dbr_time_short  tshrtval;
	struct dbr_time_float  tfltval;
	struct dbr_time_enum   tenmval;
	struct dbr_time_char   tchrval;
	struct dbr_time_long   tlngval;
	struct dbr_time_double tdblval;
    } dbrVal;			/* receive buffer for scalar item */
} AR_EL;

/*/subhead AR_REQ-------------------------------------------------------------
* ARCHIVE REQUEST related declarations
*
*	Archive requests normally reside on the main request list, which
*	is a doubly threaded list.  While on this list, a request's
*	methods are monitored, so that the request can be activated,
*	inactivated, and sampled.
*
*	While on the main request list, a request's status will usually be
*	either AR_REQ_INACTIVE or AR_REQ_ACTIVE.  When a request is
*	added to the main list, its status will be AR_REQ_CONN_OK; such
*	requests have their status changed to AR_REQ_INACTIVE and are then
*	added to the activation method queue.  The request's
*	activation method is monitored until the activation condition
*	succeeds.  When the activation method succeeds, the request's
*	status is changed to active and the inactivation and sample methods
*	are monitored.  When the inactivation method succeeds, the request
*	status is changed back to inactive and the cycle repeats.
*
*  disconnect and re-connect
*
*	When Channel Access reports a disconnect for one of the channels
*	in a request (either in an element or in a method), the request
*	status is changed to AR_REQ_CONN_CA_BAD.  The next time the main
*	request list is checked, the request will be removed from the
*	main list and added to the connect list.  While on the connect list,
*	a check is periodically made to see if all the channels are
*	connected.  When all the channels in the request are connected, the
*	status is changed to AR_REQ_CONN_CA_OK.
*
*	For requests with `by channel' sampling, checks are made that the
*	necessary connections to the channel data files exist.  As
*	required, channel descriptors are opened into the file.  If the
*	elements are all properly connected, the status is set to
*	AR_REQ_CONN_CF_OK; otherwise, to AR_REQ_CONN_CF_BAD.
*
*	For requests other than `by channel' sampling, the status is set
*	to AR_REQ_CONN_OK.
*
*	The request will remain on the connect list until its status is
*	AR_REQ_CONN_OK or AR_REQ_CONN_CF_OK.  The next time the connect
*	list is checked, the request will be moved to the main request list.
*
*  enabling a request
*
*	The usual way for a request to "come to life" begins when a
*	request skeleton containing the request's name is built and
*	placed on the read list with status of AR_REQ_CONN_NONE.  When the
*	read list is checked, requests with CONN_NONE status are read from
*	their request files, their channels are queued with Channel
*	Access for connect, and their status is set to AR_REQ_CONN_CA_BAD.
*	When all the channels for a request are connected, its status is
*	changed to AR_REQ_CONN_CA_OK.
*
*	Once the status has changed to AR_REQ_CONN_CA_OK, some final
*	validation checks are done.  If any errors are encountered at any
*	point in this process, the request is removed from the read list
*	and "thrown away".
*
*	Requests with `by channel' sampling get some additional processing.
*	The appropriate channel data file(s) is (are) opened, and each
*	element is opened to its stream(s) in the file(s).  When this
*	process is successfully completed, the request status will be
*	AR_REQ_CONN_CF_OK.
*
*	Requests with other than `by channel' sampling have their status
*	set immediately to AR_REQ_CONN_OK.
*
*	The request will remain on the read list until its status is
*	AR_REQ_CONN_OK or AR_REQ_CONN_CF_OK.  The next time the read
*	list is checked, the request will be moved to the main request list.
*	(As mentioned above, if an error is encountered, the request is
*	removed from the read list and "thrown away".)
*
*	During the time that the request is being read and checked, its
*	.readQ flag is set.  Although the preceding discussion
*	implies that the request is always on the read list while it is
*	being read, this is an oversimplification.  In actuality, the
*	request will spend part of its time on the connect list.  The
*	.readQ flag lets the connection software know that the
*	request is to be moved to the read list when connection activities
*	are complete.  When read activities are complete and the request
*	is to be moved to the main request list, the .readQ flag is
*	reset and the .mainQ flag is set.
*
*  disabling a request
*
*	The .disable flag is used when a request is to be disabled.  If
*	set, then the request is to be disabled as soon as possible.  The
*	request can be in a state, such as sampling for sample set output,
*	such that disabling must be deferred.  Some processing, such as
*	the aforementioned sampling, are inhibited if the .disable flag
*	is set; setting the flag after the processing begins, however,
*	allows the processing to proceed to completion.
*
*  request lists
*
*	One important aspect of handling requests is that of handling their
*	"residence" on doubly threaded lists.  If processing requests on a
*	list were to be handled in one task and moving requests between
*	lists were to be handled for a different task, it would be possible
*	to "side-track" the processing task if the request being processed
*	were moved to a different list.  (This would happen since the forward
*	and backward links are stored in the request.)  To avoid such
*	problems, it is important that all list traversing and all operations
*	adding and removing requests from lists be done in a single task.
*
* BUGS
* o	need a lock in each request to be used for element list
*	activities which need to be interlocked
* o	a request's methods "start from scratch" each time the request is
*	added to the main list; it might be desirable to preserve a
*	request's method state across a channel connect/disconnect.
*---------------------------------------------------------------------------*/

#define ArReqStatusToText(status) \
	(glArReqStatText[(status<0||status>=AR_REQ_PAST)?AR_REQ_PAST:status])

#ifndef AR_TEXT_GLBLSOURCE
    extern char *glArReqStatText[];
#else
    char *glArReqStatText[] = {
     /* AR_REQ_UNDEF        */    "undefined",
     /* AR_REQ_CONN_NONE    */    "disconnected",
     /* AR_REQ_CONN_CA_BAD  */    "partly connected to CA",
     /* AR_REQ_CONN_CA_OK   */    "fully connected to CA",
     /* AR_REQ_CONN_CF_BAD  */    "not fully connected to CF",
     /* AR_REQ_CONN_CF_OK   */    "fully connected to CF",
     /* AR_REQ_CONN_OK      */    "fully connected",
     /* AR_REQ_INACTIVE     */    "inactive",
     /* AR_REQ_ACTIVE       */    "active",
     /* AR_REQ_DISABLE      */    "disable",
     /* AR_REQ_PAST         */    "illegal"
    };
#endif

enum arReqStat {
    AR_REQ_UNDEF,	/* MUST BE ZERO!!! */
    AR_REQ_CONN_NONE,
    AR_REQ_CONN_CA_BAD,
    AR_REQ_CONN_CA_OK,
    AR_REQ_CONN_CF_BAD,
    AR_REQ_CONN_CF_OK,
    AR_REQ_CONN_OK,
    AR_REQ_INACTIVE,
    AR_REQ_ACTIVE,
    AR_REQ_DISABLE,

    AR_REQ_PAST		/* last legal + 1 */
};

typedef struct arSt {
    enum arItem	itemType;	/* type of item in request: AR_ITEM_STAT_xxx */
    struct arReq *pReq;		/* ptr to req containing this structure */
    chid	pCh;		/* pointer to chid for channel */
    char	name[AR_NAME_DIM];/* channel name */
} AR_STAT;			/* status channel information */

typedef struct arReq {
    struct arReq *pNext;	/* pointer to next request, or NULL */
    struct arReq *pPrev;	/* pointer to previous request, or NULL */
    long	sampleCount;	/* count%1E9 of sample events since
				activation; doesn't include monitor samples */
    AR_METH	act;		/* activation method info */
    AR_METH	inact;		/* inactivation method info */
    AR_METH	samp;		/* sample method info */
    AR_OUT	output;		/* output info */
    AR_EL	*pElListHead;	/* pointer to first element on list */
    AR_EL	*pElListTail;	/* pointer to last element on list */
    enum arReqStat status;	/* request status: AR_REQ_xxx */
    struct arSt	reqStChan;	/* request status channel */
    struct arSt	sampStChan;	/* sample status channel */
    unsigned int readQ:1;	/* req belongs on read Q when conn complete */
    unsigned int mainQ:1;	/* req belongs on main Q when conn complete */
    unsigned int disable:1;	/* request is to be disabled */
    unsigned int busy:1;	/* request is presently "busy" */
    short	connPendCount;	/* count of pending connections */
    short	elPendCount;	/* count of pending operations for elements */
    char	name[AR_NAME_DIM];	/* request name */

} AR_REQ;

/*/subhead AR_CONTEXT---------------------------------------------------------
* AR_CONTEXT
*
* DESCRIPTION
*---------------------------------------------------------------------------*/

typedef struct ar_ctx {
    char	*progName;	/* program name (up to 6 chars) */
    jmp_buf     sigEnv;         /* environment for longjmp at signal time */
    int         listenSock;     /* fd for server's "listen" socket */
    int         inUse;          /* 1 says "presently engaged" by a client */
    int         inUseInit;      /* 1 says in-use init has been done */
    char        *inUseName;     /* name of client's host */
    int         clientSock;     /* fd for socket to client */
    int         reopenIO;       /* std... should be re-directed */
    int         reopenIOoption; /* 0,1,2; option for arFreopenAllStd */
    int         oldStdinFd;     /* original fd for stdin, before redirect */
    int         oldStdoutFd;    /* original fd for stdout, before redirect */
    int         oldStderrFd;    /* original fd for stderr, before redirect */
    char        *oldPrompt;	/* prompt prior to socket connect */
    int		stop;		/* 1 says stop operations */
    AR_REQ	*pReqMainHead;	/* request list head, main processing list */
    AR_REQ	*pReqMainTail;	/* request list tail, main processing list */
    AR_REQ	*pReqConnHead;	/* request list head, for CA & CF connect */
    AR_REQ	*pReqConnTail;	/* request list tail, for CA & CF connect */
    AR_REQ	*pReqReadHead;	/* request list head, for req file read */
    AR_REQ	*pReqReadTail;	/* request list tail, for req file read */
    AR_METH	*pMethClkHead;	/* clock-based method list head */
    AR_METH	*pMethClkTail;	/* clock-based method list tail */
    AR_METH	*pMethEvHead;	/* event method list head */
    AR_METH	*pMethEvTail;	/* event method list tail */
    AR_METH	*pMethMiscHead;	/* miscellaneous method list head */
    AR_METH	*pMethMiscTail;	/* miscellaneous method list tail */
    AR_CF_DESC	*pChanFileDesc;	/* pointer to channel file descriptor */
} AR_CTX;

/*/subhead -------------------------------------------------------------------
* ARCHIVE LIBRARY related information
*---------------------------------------------------------------------------*/

#ifndef AR_TEXT_GLBLSOURCE
    extern char *glArStatText[];
#else
    char *glArStatText[] = {
     /* AR_S_OK        */    "success",
     /* AR_S_ERROR     */    "error",
     /* AR_S_NOMEM     */    "unable to get a block of memory",
     /* AR_S_CAERR     */    "non-NORMAL status from ca_xxx",
     /* AR_S_GETILL    */    "ca_get illegal--element not CONNOK or GETDONE",
     /* AR_S_NULLEL    */    "NULL supplied as element",
     /* AR_S_OPENFAIL  */    "ca_search failed",
     /* AR_S_GETFAIL   */    "ca_get failed",
     /* AR_S_OPSPEND   */    "operation illegal--elPendCount != 0",
     /* AR_S_PUTILL    */    "ca_put illegal--element not OPEN or MONITOR",
     /* AR_S_FOPENERR  */    "error opening file",
     /* AR_S_DUPCHAN   */    "duplicate channel not allowed",
     /* AR_S_SYNERR    */    "syntax error",
     /* AR_S_NOTIDLE   */    "sample skipped--output status not IDLE",
     /* AR_S_METHREQILL*/    "method illegal--req. stat. is UNDEF",
     /* AR_S_TIMEDOUT  */    "'pend' timed out",
     /* AR_S_MAXNAMELEN */   "name longer than AR_NAME_DIM",
     /* AR_S_CFOPENERR */    "non-NORMAL status from arCFOpen",
     /* AR_S_CFDESCERR */    "non-NORMAL status from arCFChanOpen",
     /* AR_S_PAST      */    "illegal AR status code"
    };
#endif

/* WARNING!!! no AR_S_xxx symbol can have a value of 0, or else arLogMessage
*		won't work properly */

#define AR_S_OK          AR_F_SUCCESS(0)
#define AR_S_ERROR          AR_F_INFO(1)
#define AR_S_NOMEM         AR_F_FATAL(2)
#define AR_S_CAERR         AR_F_FATAL(3)
#define AR_S_GETILL        AR_F_FATAL(4)
#define AR_S_NULLEL        AR_F_FATAL(5)
#define AR_S_OPENFAIL       AR_F_WARN(6)
#define AR_S_GETFAIL        AR_F_WARN(7)
#define AR_S_OPSPEND        AR_F_WARN(8)
#define AR_S_PUTILL        AR_F_FATAL(9)
#define AR_S_FOPENERR      AR_F_WARN(10)
#define AR_S_DUPCHAN       AR_F_WARN(11)
#define AR_S_SYNERR        AR_F_WARN(12)
#define AR_S_NOTIDLE      AR_F_FATAL(13)
#define AR_S_METHREQILL   AR_F_FATAL(15)
#define AR_S_TIMEDOUT	   AR_F_INFO(16)
#define AR_S_MAXNAMELEN    AR_F_INFO(17)
#define AR_S_CFOPENERR     AR_F_INFO(18)
#define AR_S_CFDESCERR     AR_F_INFO(19)

#define AR_S_PAST			20	/* last legal code + 1 */

#define ArStatusToCode(status) \
	((status>>4<0||status>>4>=AR_S_PAST)?AR_S_PAST:status>>4)

#define ArStatusToText(status) (glArStatText[ArStatusToCode(status)])


/* STATUS CODES, support for actual definitions
*    low order nibble indicates type of error.
*    odd values are success
*    even values are failure
*/
#define AR_K_SUCCESS 1
#define AR_K_INFO    3
#define AR_K_WARN    0
#define AR_K_FATAL   6

#define AR_F_INFO(code)    (code<<4 | AR_K_INFO)
#define AR_F_SUCCESS(code) (code<<4 | AR_K_SUCCESS)
#define AR_F_WARN(code)    (code<<4 | AR_K_WARN)
#define AR_F_FATAL(code)   (code<<4 | AR_K_FATAL)


/*----------------------------------------------------------------------------
* keywords
*---------------------------------------------------------------------------*/
#ifndef AR_TEXT_GLBLSOURCE
    extern char glArKwName[];
    extern char glArKwDesc[];
    extern char glArKwAct[];
    extern char glArKwInact[];
    extern char glArKwSamp[];
    extern char glArKwReqStatus[];
    extern char glArKwSampStatus[];
    extern char glArKwOut[];
    extern char glArKwEvery[];
    extern char glArKwTimes[];
    extern char glArKwEndHeader[];
    extern char glArKwEndData[];
#else
    char glArKwEvery[]=     "every";
    char glArKwTimes[]=     "times";
    char glArKwName[]=          "NAME:";
    char glArKwDesc[]=          "DESC:";
    char glArKwAct[]=           "ACT:";
    char glArKwInact[]=         "INACT:";
    char glArKwSamp[]=          "SAMPLE:";
    char glArKwReqStatus[]=     "REQ_STATUS:";
    char glArKwSampStatus[]=    "SAMP_STATUS:";
    char glArKwOut[]=           "OUTPUT:";
    char glArKwEndHeader[]=     "%endHeader%";
    char glArKwEndData[]=       "%endData%";
#endif

#endif
