/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
/* 
 * arrAppInit.c --
 *
 *	Provides a default version of the Tcl_AppInit procedure for
 *	use in wish and similar Tk-based applications.
 *
 */

#ifndef lint
static char rcsid[] = "/usr/local/cvsroot/epics/extensions/src/ar/arrtk/arrAppInit.c,v 1.1.1.1 1995/09/01 01:34:16 epics Exp SPRITE (Berkeley)";
#endif /* not lint */

#include "tk.h"
#include <tk.h>
#include <blt.h>

/*
 * The following variable is a special hack that is needed in order for
 * Sun shared libraries to be used for Tcl.
 */

#ifdef NEED_MATHERR
extern int matherr();
int *tclDummyMathPtr = (int *) matherr;
#endif



/*
 * The following variable is a special hack that allows applications
 * to be linked using the procedure "main" from the Tk library.  The
 * variable generates a reference to "main", which causes main to
 * be brought in from the library (and all of Tk and Tcl with it).
 */

extern int Arr_FileList();
extern int Arr_GetData();
extern int Arr_TsAddSec();
extern int Arr_TsCmpStamps();
extern int Arr_DiffAsSec();
extern int Arr_DiffAsStamp();

/*
extern int main();
int *tclDummyMainPtr = (int *) main;
*/

/*
 *----------------------------------------------------------------------
 *
 * main --
 *
 *  This is the main program for the application.
 *
 * Results:
 *  None: Tk_Main never returns here, so this procedure never
 *  returns either.
 *
 * Side effects:
 *  Whatever the application does.
 *
 *----------------------------------------------------------------------
 */

extern int Tcl_AppInit _ANSI_ARGS_((Tcl_Interp *interp));


int main(argc,argv)
int argc;
char *argv[];
{
    Tk_Main(argc,argv,Tcl_AppInit);
    return 0;           /* Needed only to prevent compiler warning. */
}


/*
 *----------------------------------------------------------------------
 *
 * Tcl_AppInit --
 *
 *	This procedure performs application-specific initialization.
 *	Most applications, especially those that incorporate additional
 *	packages, will have their own version of this procedure.
 *
 * Results:
 *	Returns a standard Tcl completion code, and leaves an error
 *	message in interp->result if an error occurs.
 *
 * Side effects:
 *	Depends on the startup script.
 *
 *----------------------------------------------------------------------
 */

int
Tcl_AppInit(interp)
    Tcl_Interp *interp;		/* Interpreter for application. */
{
    Tk_Window main;

    /*
     * Call the init procedures for included packages.  Each call should
     * look like this:
     *
     * if (Mod_Init(interp) == TCL_ERROR) {
     *     return TCL_ERROR;
     * }
     *
     * where "Mod" is the name of the module.
     */

    if (Tcl_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }

    if (Tk_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }

    if (Blt_Init(interp) == TCL_ERROR) {
	return TCL_ERROR;
    }

    main = Tk_MainWindow(interp);

    /*
     * Call Tcl_CreateCommand for application-specific commands, if
     * they weren't already created by the init procedures called above.
     */

    Tcl_CreateCommand(interp, "arr_fileList", Arr_FileList,
		(ClientData *) NULL, (Tcl_CmdDeleteProc *) NULL);

    Tcl_CreateCommand(interp, "arr_getData", Arr_GetData,
		(ClientData *) NULL, (Tcl_CmdDeleteProc *) NULL);

    Tcl_CreateCommand(interp, "arr_tsAddSec", Arr_TsAddSec,
		(ClientData *) NULL, (Tcl_CmdDeleteProc *) NULL);

    Tcl_CreateCommand(interp, "arr_tsCmpStamps", Arr_TsCmpStamps,
		(ClientData *) NULL, (Tcl_CmdDeleteProc *) NULL);

    Tcl_CreateCommand(interp, "arr_diffAsSec", Arr_DiffAsSec,
		(ClientData *) NULL, (Tcl_CmdDeleteProc *) NULL);

    Tcl_CreateCommand(interp, "arr_diffAsStamp", Arr_DiffAsStamp,
		(ClientData *) NULL, (Tcl_CmdDeleteProc *) NULL);

    /*
     * Specify a user-specific startup file to invoke if the application
     * is run interactively.  Typically the startup file is "~/.apprc"
     * where "app" is the name of the application.  If this line is deleted
     * then no user-specific startup file will be run under any conditions.
     */

    /*tcl_RcFileName = "~/.wishrc"; */
    return TCL_OK;
}
