/*************************************************************************\
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory.
* Copyright (c) 2002 The Regents of the University of California, as
* Operator of Los Alamos National Laboratory.
* This file is distributed subject to a Software License Agreement found
* in the file LICENSE that is included with this distribution. 
\*************************************************************************/
#ifndef INC_sydDefs_h
#define INC_sydDefs_h
/*	sydDefs.h,v 1.3 2002/08/02 15:38:00 jba Exp
 *	Author:	Roger A. Cole
 *	Date:	12-04-90
 *
 *	Experimental Physics and Industrial Control System (EPICS)
 *
 * include options
 *	-DSYD_PRIVATE	makes structures visible--use only by sydXxx routines
 *	-DSYD_PRIVATE_DATA defines global text fields--use only by sydSubr.c
 */
/*+/mod***********************************************************************
* TITLE	sydDefs.h - definitions for synchronous data routines
*
* DESCRIPTION
*
*-***************************************************************************/
#ifndef INC_genDefs_h
#   include <genDefs.h>
#endif
#ifndef INCLcadefh
#   include <cadef.h>
#endif
#include <errMdef.h>	/* get M_syd for this subsystem's `number' */
/*-----------------------------------------------------------------------------
* some MACROS for handling synchronous data structures
*----------------------------------------------------------------------------*/
#define NEXT_INBUF(pSChan, sub) (((sub)+1 < pSChan->nInBufs) ? (sub)+1 : 0)


/*/subhead SYD_ATTR------------------------------------------------------------
*
*----------------------------------------------------------------------------*/
typedef enum sydAttr {
    SYD_ATTR_DEADBAND,
    SYD_ATTR_MON_FN,
    SYD_ATTR_MON_ARG,
    SYD_ATTR_USE_STATS,
    SYD_ATTR_USE_MEANS,
} SYD_ATTR;

/*/subhead SYD_STATS-----------------------------------------------------------
*
*----------------------------------------------------------------------------*/
typedef struct sydStats {
    double	mean;
    double	stdDev;
} SYD_STATS;

/*/subhead SYD_TEST---------------------------------------------------------
* SYD_TEST
*
*    .type	type of triggering to do
*    .cond	trigger condition for conditional trigger types
*    .pSChan	sync channel ptr for types of triggering which use channels
*    .compVal	is the value for comparison trigger types
*----------------------------------------------------------------------------*/
enum sydTestType {
    SYD_TST_IF,
};
enum sydTestCond {
    SYD_TSTC_UNDEF,
    SYD_TSTC_CHANGE,
    SYD_TSTC_EQ,
    SYD_TSTC_NE,
    SYD_TSTC_GT,
    SYD_TSTC_GE,
    SYD_TSTC_LT,
    SYD_TSTC_LE,
    SYD_TSTC_PAST,
};
#ifndef SYD_PRIVATE_DATA
    extern char *glSydTestText[];
#else
    char *glSydTestText[] = {
	/* SYD_TSTC_UNDEF      */ "undefined",
	/* SYD_TSTC_CHANGE     */ "change",
	/* SYD_TSTC_EQ         */ "==",
	/* SYD_TSTC_NE         */ "!=",
	/* SYD_TSTC_GT         */ ">",
	/* SYD_TSTC_GE         */ ">=",
	/* SYD_TSTC_LT         */ "<",
	/* SYD_TSTC_LE         */ "<=",
	/* SYD_TSTC_PAST       */ "illegal",
    };
#endif

typedef struct sydTest {
    enum sydTestType type;
    enum sydTestCond cond;
    struct sydChan *pSChan;
    double	dblVal;
    long	lngVal;
    char	strVal[db_strval_dim];
} SYD_TEST;

/*/subhead SYD_CHAN--------------------------------------------------------
*
* input buffer status should not be confused with the status of input data.
* In general, the status of the data is a part of the structure containing
* the data.  The data structures are of type DBR_TIME_xxx, which has
* .status and .severity items.  The input buffer status codes are:
*
*    SYD_B_EMPTY	the buffer is empty of data (and status)
*    SYD_B_FULL		the buffer contains data with a valid time stamp
*    SYD_B_RESTART	similar to SYD_B_FULL, except the value restarts
*		a "run" following missing data.  This status is used at
*		the beginning of file and when an archive request is
*		activated.  (For "retrievals" for real-time data, this
*		is used following either the initial connection or a
*		resumed connection after a disconnect.)
*    SYD_B_SNAP_BEGIN	similar to SYD_B_FULL, except the value is the
*		first sample in a snapshot.
*    SYD_B_SNAP_END	similar to SYD_B_SNAP_BEGIN, except the value is the
*		last sample in a snapshot.
*    SYD_B_SNAP_SINGLE	similar to SYD_B_SNAP_BEGIN, except the value is the
*		only sample in a snapshot.
*    SYD_B_EOF		channel's stream is at end of file
*    SYD_B_MISSING	there is no data for the channel.  This status
*		indicates the beginning of a "hiatus" in data for the
*		channel, as between inactivate and activate for an
*		archive request or between disconnect and reconnect for
*		a real-time channel.  This status is different from
*		SYD_B_EMPTY because this status has a valid time stamp,
*		which is important in deciding how to construct synchronous
*		samples.
*    SYD_B_SAMPLED	the buffer has valid data which has been used
*		in at least one sample.
*----------------------------------------------------------------------------*/
enum sydBStatus {
    SYD_B_EMPTY,
    SYD_B_MISSING,
    SYD_B_RESTART,
    SYD_B_FULL,
    SYD_B_SAMPLED,
    SYD_B_SNAP_BEGIN,
    SYD_B_SNAP_END,
    SYD_B_SNAP_SINGLE,
    SYD_B_EOF
};
enum sydChanStep {
    SYD_ST_UNDEF,
    SYD_ST_STEP,
    SYD_ST_SMOOTH,
    SYD_ST_PAST
};
enum sydChanSync {
    SYD_SY_UNDEF,
    SYD_SY_NONF,
    SYD_SY_FILLED,
    SYD_SY_PAST
};

struct sydChanFlags {
    unsigned int ok : 1;	/* value is OK; often with other flags */
    unsigned int restart : 1;	/* value restarts a "run"; often with other
				flags */
    unsigned int snapstart : 1;	/* value is the first in a snapshot */
    unsigned int snapend : 1;	/* value is the last in a snapshot */
    unsigned int filled : 1;	/* value filled from prior value */
    unsigned int missing : 1;	/* no value for time stamp; filled values
				will not have this flag set.  This will
				often be set in conjunction with other
				flags, such as EOF */
    unsigned int eof : 1;	/* end of file */
    unsigned int discon : 1;	/* channel was disconnected */
    unsigned int inact : 1;	/* archive request was inactivated */
};

/*/subhead SYD_CHAN------------------------------------------------------------
* SYD_CHAN
*----------------------------------------------------------------------------*/
typedef struct sydChan {
    struct sydChan *pPrev;      /* link to previous channel */
    struct sydChan *pNext;      /* link to next channel */
    struct sydSpec *pSspec;	/* pointer to synchronous data spec */
    void	*pHandle;
    evid	evid;
    short	conn;		/* 1 if connected at least once */
    short	discon;		/* 1 if currently disconnected */
    short	gotGr;
    short	dataChan;
    short	testChan;
    short	restart;
#define SYD_CHAN_MAX_IN 10
    short	nInBufs;
    short	firstInBuf;
    short	lastInBuf;
    short	sampInBuf;
    short	reused;
    union db_access_val	*pInBuf[SYD_CHAN_MAX_IN];
    enum sydBStatus inStatus[SYD_CHAN_MAX_IN];
    double	maxDataVal;
    double	minDataVal;
    short	minMaxNeedInit;
    double	restrictMaxDataVal;
    double	restrictMinDataVal;
    SYD_STATS	*pStats;
    void	*pData;
    UCHAR	*pDataAlStat;
    UCHAR	*pDataAlSev;
    char	*pDataCodeL;
    char	*pDataCodeR;
    struct sydChanFlags *pFlags;
    char        name[db_name_dim];
    char	EGU[db_units_dim];
    char	label[120];	/* label using chanName and units */
    short	precision;	/* number of decimal places to print;
				0 for integers, -1 for ENUM */
    short	useExp;		/* 1 for "use exponential notation" */
    unsigned short elCount;
    chtype      dbrType;
    chtype	dbfType;
    enum sydChanStep step;
    enum sydChanSync sync;
    short	isRVAL;		/* 1 if channel is .RVAL */
    union db_access_val	grBuf;
    chtype	dbrGrType;
    void	*pArg;
} SYD_CHAN;
/*-----------------------------------------------------------------------------
*       A synchronous data channel descriptor provides for:
*	o   buffering data during "acquisition"
*	o   preparing "synchronous samples"
*	o   building an accumulation of "synchronous samples" (perhaps in
*	    conjunction with the select specification from the Sspec)
*
* SYD_CHAN_MAX_IN
*	This symbol limits the number of input buffers for saving data
*	while trying to build a synchronous sample.  When retrieving
*	from AR "by channel" streams, 2 input buffers are required.
*	For AR "sample set" streams, 2 input buffers are required.
*	For pfoPlot "streams", 2 input buffers are required.
*	When retrieving from real-time channels, this number, in
*	conjunction with the beam rep rate, indicates the delay which
*	will be tolerated.  A value of 10 with a rep rate of 10 indicates
*	that a 1 second delay can be handled.
*
* interact with the "source"
*
*	pSspec->pFunc	points to a function which is called to open
*			and close the source and to acquire data.  See
*			SYD_SPEC for more information.
*
*	.pHandle	a "handle" to be used by pFunc() for interacting
*			with the "source" for this channel.  The nature
*			of this handle depends on Cset type:
*			o  for SYD_TY_CF pHandle is (AR_CHAN_DESC *)
*			o  for SYD_TY_SSF pHandle is not used
*			o  for SYD_TY_CA pHandle is (chid)
*	.evid		for use only with SYD_TY_CA; (evid)
*	.gotGr		for use only with SYD_TY_CA; 1 says have gr info
*	.restart	for use only with SYD_TY_CA; 1 says restart a run
*
* acquire data from the "source"
*
*	Data acquired from the source is stored in input buffers.  There
*	is a fixed size array of pointers to input buffers; the actual
*	number of input buffers which are available to hold data is
*	available in nInBufs.  There are several arrays which parallel
*	the array of input buffer pointers.
*
*	.nInBufs	the number of input buffers available
*	.firstInBuf	subscript for oldest input buffer containing data.
*	.lastInBuf	subscript for last input buffer to receive data.
*	.sampInBuf	subscript of input buffer which is to be part
*			of a synchronous sample; if < 0, this channel doesn't
*			contribute to the sample.
*	.reused		if 1, the value indicated by .sampInBuf has been
*			used in previous samples; else 0  .
*	.pInBuf[]	pointers to input buffers
*	.inStatus[]	statuses for the input buffers; one of SYD_B_xxx
*
* synchronous sample set storage
*	.maxDataVal	maximum value in the synchronous set
*	.minDataVal	minimum value in the synchronous set
*	.restrictMaxDataVal maximum value for the restricted range
*	.restrictMinDataVal minimum value for the restricted range
*	.pData		pointer to array of data values.  For array channels,
*			a "data value" in *pData is actually an array of
*			dimension [.elCount] of values.
*	.pDataAlStat	pointer to array of data alarm status codes
*	.pDataAlSev	pointer to array of data alarm severity codes
*	.pFlags		pointer to array of flag structures
*	.pDataCodeL	pointer to array of left-hand status code char's
*	.pDataCodeR	pointer to array of right-hand status code char's
*
* channel characteristics
*	.name		name of the channel (as entered by user)
*	.EGU		engineering units text
*	.label		a label consisting of name and units
*	.precision	number of dec. places to print; 0 for int, -1 for ENUM
*	.elCount	array element count for channel
*	.dbrType	form in which data is to be acquired and stored
*	.step		indicates how the channel's value changes:
*			SYD_ST_STEP	changes as a step function
*			SYD_ST_SMOOTH	changes smoothly between values
*	.sync		indicates if channel is:
*			SYD_SY_NONF	not filled, or
*			SYD_SY_FILLED	filled
*	.grBuf		a (union db_access_val) containing graphics info
*	.dbrGrType	type of graphics buffer--one of DBR_GR_xxx
*----------------------------------------------------------------------------*/

/*/subhead SYD_SPEC-------------------------------------------------------
* SYD_SPEC
*----------------------------------------------------------------------------*/
enum sydFuncCode {
    SYD_FC_INIT,
    SYD_FC_OPEN,
    SYD_FC_READ,
    SYD_FC_READ_SYNC,
    SYD_FC_POSITION,
    SYD_FC_CLOSE,
    SYD_FC_FILEINFO,
    SYD_FC_STOP,
    SYD_FC_WRAPUP
};
enum sydType {
    SYD_TY_CF,		/* AR channel file */
    SYD_TY_SSF,		/* AR sample set file */
    SYD_TY_CA,		/* Channel Access */
};

typedef struct sydSpec {
    long	(*pFunc)();
    enum sydType type;
    void	*pHandle;
    double      sampleSec;      /* `X' coordinate, as seconds past
                                           the reference time of the
                                           entire sync sample set */
    short	partial;	/* partial sample flag */
    SYD_TEST	*pAccept;	/* test for accepting sync samples, or NULL */
    SYD_CHAN	*pChanHead;     /* pointer to head of channel list */
    SYD_CHAN	*pChanTail;     /* pointer to tail of channel list */
    TS_STAMP	sampleTs;       /* time for sync sample */
    TS_STAMP	priorTs;        /* time for prior sync sample */
    TS_STAMP	refTs;          /* reference time for entire sync sample set */
    int		reqCount;       /* number of sets to acquire */
    int		sampleCount;    /* number of sets acquired */
    short	chanCount;      /* count of channels selected */
    short	disconCount;	/* count of channels disconnected */
    short	eofCount;       /* count of channels with EOF */
    short	needGrCount;	/* count of channels needing graphics info */
    long	dataDim;
    long	lastData;
    long	firstData;
    long	restrictLastData;
    long	restrictFirstData;
    TS_STAMP	restrictRefTs;
    double	restrictDeltaSecSubtract;
    double	*pDeltaSec;
    TS_STAMP	*pTimeStamp;
    char	*pPartial;
    long	statDim;
    long	statCount;
    double	*pStatDeltaSec;
    TS_STAMP	*pStatTimeStamp;
    long	*pStatPopCount;
    short	nInBufs;
    unsigned long roundNsec;	/* nano-second rounding interval */
    unsigned long deadband;	/* deadband flag for ca_add_event */
    short	useStats;	/* flag indicating to use stats, not data */
    short	useMeans;	/* flag indicating to use only means, used
				in conjunction with useStats */
    void	(* monFn)();	/* monitor callback function */
    void	*monArg;	/* monitor callback argument */
} SYD_SPEC;

/*------------------------------------------------------------------------
*
*       A synchronous set spec is the `master structure' which is used for
*       accessing data.  The SYD_SPEC is created empty, and then filled
*       in with additional calls.
*
* interact with the "source"
*	.pFunc		points to a function which is called to open
*			and close the source and to acquire data.
*			(In some cases, data is stored by some outside
*			agent, rather than by calling pFunc().)  When
*			pFunc() is called, it is passed a function
*			code telling it what to do:
*			SYD_FC_INIT	initialize the "source"
*			SYD_FC_OPEN	"open" a channel
*			SYD_FC_READ	"read" a data item for a channel (or
*				start monitoring for Channel Access)
*			SYD_FC_STOP	stop monitoring (for Channel Access)
*			SYD_FC_CLOSE	"close" a channel
*			SYD_FC_WRAPUP	wrapup the "source"
*
*			sydCFFunc()	for "by channel" data
*			sydSSFFunc()	for "sample set" data
*	.type		type of source:
*			SYD_TY_CF   "by channel"
*			SYD_TY_SSF  "sample set"
*
*	.pHandle	an argument to be used by pFunc().  Typically,
*			this will be an AR_CF_DESC, a FILE *, or NULL.
*
* "time base" storage for synchronous set
*	.dataDim	the number of data "slots" available
*	.firstData	subscript of first "slot" item containing data.
*	.lastData	subscript of last "slot" item containing data. All
*			"slots" from [firstData] through [.lastData]
*			have data (wrapping around, if needed).
*	.restrictFirstData  subscript of first sample in restricted set
*	.restrictLastData  subscript of last sample in restricted set
*	.restrictRefTs	time stamp (probably rounded) for start of restricted
*			set of samples
*	.restrictDeltaSecSubtract number of seconds to subtract from the
*			sample delta seconds to get time past restrictRefTs
*	.pDeltaSec[]	pointer to array of delta times past first sample
*	.pTimeStamp[]	pointer to array of time stamps for samples
*	.pPartial[]	pointer to array of partial sample flags: 1 indicates
*			indicates a partial sample--one or more
*			channels in the sample has a status flag which is
*			not one of the SYD_VAL_OKxxx flags.
*----------------------------------------------------------------------------*/


/*/subhead status codes-------------------------------------------------------
*		S T A T U S   C O D E S
*
*	
*    Status codes of the form S_ts_briefMessage are used with the AR access
*    library routines.
*
*    The macro SydStatusToText(stat) can be used to obtain a text string
*    corresponding to a status code.
*
*----------------------------------------------------------------------------*/
#define S_syd_OK		(M_syd|0| 0<<1) /* success */
#define S_syd_noMem		(M_syd|1| 1<<1) /* couldn't malloc memory */
#define S_syd_chanNotFound	(M_syd|1| 2<<1) /* couldn't find channel */
#define S_syd_partial		(M_syd|1| 3<<1) /* partial retrieval set */
#define S_syd_EOF		(M_syd|1| 4<<1) /* end of file */
#define S_syd_ERROR		(M_syd|1| 5<<1) /* other error */
#define S_syd_noDataNow		(M_syd|1| 6<<1) /* no data available now */
#define S_syd_chanNotConn	(M_syd|1| 7<<1) /* chan not connected to CA */

#define SYD_STAT_S_PAST		 8	/* one past last legal code */

#define SydStatusToIndex(status) \
	(  ((status&0xffff0000)!=M_syd)  \
		? SYD_STAT_S_PAST  \
		: (  (((status&0xffff)>>1)>=SYD_STAT_S_PAST)  \
		     ? SYD_STAT_S_PAST  \
		     : ((status&0xffff)>>1)  \
		  )  \
	)

#define SydStatusToText(status) \
	(glSydStatText[SydStatusToIndex(status)])

#ifndef SYD_PRIVATE_DATA
    extern char *glSydStatText[];
#else
    char *glSydStatText[] = {
	/* S_syd_OK            */ "success",
	/* S_syd_noMem         */ "couldn't malloc memory",
	/* S_syd_chanNotFound  */ "couldn't find channel",
	/* S_syd_partial       */ "partial retrieval set",
	/* S_syd_EOF           */ "end of file",
	/* S_syd_ERROR         */ "error",
	/* S_syd_noDataNow     */ "no data available now",
	/* S_syd_chanNotConn   */ "channel not connected",

	/* SYD_STAT_S_PAST          */ "illegal syd status code",
    };
#endif

/*/subhead macros--------------------------------------------------------------
* macros
*----------------------------------------------------------------------------*/
#define SydChanArrayCount(pSChan) ((pSChan)->elCount)
#define SydChanDbfType(pSChan) \
	((pSChan)->discon \
	    ? TYPENOTCONN \
	    : (pSChan)->dbfType)
#define SydChanDbrType(pSChan) ((pSChan)->dbrType)
#define SydChanName(pSChan) ((pSChan)->name)
#define SydChanEGU(pSChan) ((pSChan)->EGU)
#define SydInputTs(pSspec) (pSspec)->sampleTs
#define SydInputStatus(pSChan) \
	((pSChan)->sampInBuf < 0 \
	    ? SYD_B_MISSING \
	    : (pSChan)->inStatus[(pSChan)->sampInBuf])
#define SydInputMarkAsSampled(pSChan) \
	((pSChan)->sampInBuf < 0 \
	    ? SYD_B_MISSING \
	    : ((pSChan)->inStatus[(pSChan)->sampInBuf]=SYD_B_SAMPLED))
#define SydInputValAsVoidPtr(pSChan) (dbr_value_ptr( \
    (void *)(pSChan)->pInBuf[(pSChan)->sampInBuf], SydChanDbrType(pSChan)))
#define SydInputValAsFloat(pSChan) (*(float *)SydInputValAsVoidPtr(pSChan))
#define SydInputValAsDouble(pSChan) (*(double *)SydInputValAsVoidPtr(pSChan))
#define SydInputValAsShort(pSChan) (*(short *)SydInputValAsVoidPtr(pSChan))
#define SydInputValAsLong(pSChan) (*(long *)SydInputValAsVoidPtr(pSChan))
#define SydInputValAsChar(pSChan) (*(char *)SydInputValAsVoidPtr(pSChan))
#define SydInputValAsString(pSChan) ((char *)SydInputValAsVoidPtr(pSChan))


/*/subhead prototypes----------------------------------------------------------
* prototypes
*----------------------------------------------------------------------------*/
long sydChanClose();
long sydChanConnHandler();
SYD_CHAN *sydChanFind();
long sydChanOpen(), sydChanOpen1(), sydChanOpenGR();
long sydChanPrep();
long sydClose();
void sydCopyGr();
void sydCopyVal();
long sydFileInfo();
long sydInputFetch();
long sydInputGet();
void sydInputMarkAsSampled();
void sydInputReset();
void sydInputResetSampled();
void sydInputResetKeepNewest();
void sydInputStoreInSet();
long sydInputSync();
long sydMonitorStart();
long sydMonitorStop();
long sydOpen();
long sydOpenCA();
long sydOpenCF();
long sydOpenSSF();
long sydPosition();
long sydSampleExport();
long sydSampleExportStats();
long sydSampleMean();
long sydSamplePrint();
long sydSampleWriteSSF();
long sydSampleSetAlloc();
long sydSampleSetExport();
long sydSampleSetFree();
long sydSampleSetGet();
long sydSampleSetPrint();
long sydSampleSetRestrict();
long sydSampleSetStats();
long sydSampleSetWriteSSF();
long sydSetAttr();
long sydTest();
long sydTestAddFromText();
long sydTestClose();
int sydValAsDbl();

#endif
